// arch-tag: 864bd915-6eef-4f04-83f7-185ded73681f
//

class TMyCdrEqn : public TCdrEqn<3> {

private:
 
    TFunction<3> *u,*ux,*uy,*uz,*uxx,*uyy,*uzz,*uxy,*uxz,*uyz;
    TFunction<3> *D00x, *D10y,*D20z,*D01x,*D11y,*D21z,*D02x,*D12y,*D22z;


public:

    
    TMyCdrEqn( TFloat aEpsilon = 1.0, 
               TBlockVector<TFloat> *aConvVector = NULL): TCdrEqn<3>(aEpsilon,aConvVector)
    {};

    
 
    virtual ~TMyCdrEqn () { }

    void setSolution(TFunction<3> *au,
                     TFunction<3> *aux,
                     TFunction<3> *auy,
                     TFunction<3> *auz,
                     TFunction<3> *auxx,
                     TFunction<3> *auyy,
                     TFunction<3> *auzz,
                     TFunction<3> *auxy,
                     TFunction<3> *auxz,
                     TFunction<3> *auyz )
    { 
        u=au;
        ux=aux; uy=auy; uz=auz;
        uxx=auxx; uyy=auyy; uzz=auzz;
        uxy=auxy; uxz=auxz; uyz=auyz;
    };

    void setDiffusionDerivatives(TFunction<3> *aD00x ,
                                 TFunction<3> *aD10y ,
                                 TFunction<3> *aD20z ,
                                 TFunction<3> *aD01x,
                                 TFunction<3> *aD11y,
                                 TFunction<3> *aD21z,
                                 TFunction<3> *aD02x,
                                 TFunction<3> *aD12y,
                                 TFunction<3> *aD22z)
        
    { 
        D00x=aD00x; D10y=aD10y; D20z=aD20z;
        D01x=aD01x; D11y=aD11y; D21z=aD21z;
        D02x=aD02x; D12y=aD12y; D22z=aD22z;
    };





    TFloat evalRhs(TVector <3> *v, uint i = 0 ) const{
       
        TFloat rhs=0;
        // D
        if (diffFlag==TRUE){
            if (laplaceFlag==TRUE)
                rhs = -epsilon*( uxx->eval(v)+uyy->eval(v)+uzz->eval(v));
            else{
                // 2nd derivatives
                rhs  = - (d[0][0])->eval(v)*uxx->eval(v);
                rhs -= (d[1][1])->eval(v)*uyy->eval(v);
                rhs -= (d[2][2])->eval(v)*uzz->eval(v);
              
                // mixed 2nd derivatives
                rhs -= ((d[0][1])->eval(v) + (d[1][0])->eval(v))*uxy->eval(v);
                rhs -= ((d[0][2])->eval(v) + (d[2][0])->eval(v))*uxz->eval(v);
                rhs -= ((d[1][2])->eval(v) + (d[2][1])->eval(v))*uyz->eval(v);

                // first derivatives
            
                rhs -= (D00x->eval(v)+ D10y->eval(v)+D20z->eval(v))*ux->eval(v);
                rhs -= (D01x->eval(v)+ D11y->eval(v)+D21z->eval(v))*ux->eval(v);
                rhs -= (D02x->eval(v)+ D12y->eval(v)+D22z->eval(v))*ux->eval(v);
            }            
        }// diffFlag


        if (convFlag==TRUE){
            TVector<3> B;
            evalConv(v,&B);
            TVector<3> gradU;
            gradU[0]=ux->eval(v); gradU[1]=uy->eval(v); gradU[2]=uz->eval(v);
            rhs += B*gradU;
        }

        if (reacFlag==TRUE)
            rhs += evalReac(v)*u->eval(v);
        return rhs;
   } // evalRhs
    
};





class TNeumannFunc : public TFunction<6>{
protected:
    TFunction<3> *ux;
    TFunction<3> *uy;
    TFunction<3> *uz;

public:
    TNeumannFunc(TFunction<3> *aux, TFunction<3> *auy, TFunction<3> *auz)
    { ux=aux; uy=auy;uz=auz;    };

    TFloat eval(TVector <6> *av ){
    
        TVector<3> x,n,gradU;
        for (uint i=0; i<3;i++){
            x[i]=(*av)[i];    //  first part of *av : x 
            n[i]=(*av)[i+3];  // second part of *av: n 
        }
        gradU[0]=ux->eval(&x);
        gradU[1]=uy->eval(&x);
        gradU[2]=uz->eval(&x);
        return gradU*n;
    } // eval;
};
