//
//  Project   : libnet
//  File      : basisCdrEqn3D.cc
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for pdes
//
// arch-tag: c120c839-f15d-470f-a361-049c3deaf586
//

#include "TBasisCdrEqn.hh"

// constructor	
template <int N>
TBasisCdrEqn<N>::TBasisCdrEqn ( TFloat aEpsilon, 
                                TBlockVector<TFloat> *aConvVector )
        : TConvEqn<N> ()
{ 
   epsilon = aEpsilon;
  
   if (aEpsilon==0.0){
      laplaceFlag=FALSE;
      diffFlag=FALSE;}
   else{
      laplaceFlag=TRUE;
      diffFlag=TRUE;
   }

   convVector=aConvVector;
   if (aConvVector==NULL)
      convFlag=FALSE;
   else
      convFlag=TRUE;
   
   reacFlag=FALSE;
   
}


// delete convection Vector
template <int N>
void 
TBasisCdrEqn<N>::deleteConvVector () {
   delete (convVector);
   convVector = NULL;
}


// init convVector using the convection functions
template <int N>
void 
TBasisCdrEqn<N>::initConvVector ( TDynArray< TVector <N> * > & coordVec )
{
   if (!convFlag) 
      cerr << "TBasisCdrEqn::initConvDirection : no convection defined\n"; 
   else{
      uint no[N+1];
      uint size=coordVec.getSize();
      
      for (uint i=0;i<N+1;i++) no[i]=size;
      
      // allocation of convVector (if necessary)
      if ( convVector == NULL )
         convVector = new TBlockVector<TFloat> ( N+1, no );
      
      // evaluate b  at the coordinates belonging to unknown 'k'
      // write result to convVector
      TVector<N> *b=new (TVector<N> );
      
      for ( uint k = 0; k < size; k++ ){
         evalConv(coordVec[k],b);    
         for (uint i=0;i<N;i++) 
            (*convVector)[i][k] =(*b)[i];
      } 
   }//else
}
  

template class TBasisCdrEqn<2>;
template class TBasisCdrEqn<3>;
