//
//  Project   : libnet
//  File      : TCdrEqn.cc
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for pdes
//
// arch-tag: 01691eea-c1d9-4b40-9a65-1c667c60b00e
//

#include "TCdrEqn.hh"

// constructor	
template <int N>
TCdrEqn<N>::TCdrEqn ( TFloat aEpsilon, 
                      TBlockVector<TFloat> *aConvVector ) :
   TBasisCdrEqn<N>(aEpsilon,aConvVector)
{ 

    // initialise diffusion functions 
   for (uint i=0; i<N; i++)
      for (uint j=0; j<N; j++)
         d[i][j]  = NULL;
   // if isLaplace, keep diffFlag; otherwise init diffFlag as FALSE
   if ( TBasisCdrEqn<N>::isLaplace() == TRUE )
       TBasisCdrEqn<N>::Diffusion( TRUE );
   else 
       TBasisCdrEqn<N>::Diffusion( FALSE );


   
   for (uint i=0; i<N; i++)
       b[i]  = NULL;

   c  = NULL;
   f  = NULL;
}


// evaluate Diffusion matrix
template <int N>
void 
TCdrEqn<N>::evalDiff(TVector <N> *v, TMatrix<N>  *D) const
{
   for (uint i=0; i<N;i++)
      for (uint j=0; j<N;j++){
         if(d[i][j])
            (*D)[i][j]=(d[i][j])->eval(v);
         else
         {
             cerr << "TCdrEqn<N>::evalDiff: no Diff function d["
                  << i << "][" << j << "] defined" << endl;
             exit(1);
         }// else
      }
   
}   
// evaluate convection vector
template <int N>
void 
TCdrEqn<N>::evalConv(TVector <N> *v, TVector<N>  *B) const{

   for (uint i=0; i<N;i++){
      if(b[i])
         (*B)[i]=(b[i])->eval(v);
      else
         cerr<<"TCdrEqn<N>::evalConv: no Conv function b["<<i<<"] defined "<<endl;
   }
     
}
   
// evaluate reaction 
template <int N>
TFloat 
TCdrEqn<N>::evalReac(TVector <N> *v)const{
   if (c)
      return c->eval(v);
   else{ 
      cerr<<"TCdrEqn<N>::evalReac: no reac function defined "<<endl;
      return 0;
   }
}


// evaluate rhs
template <int N>
TFloat 
TCdrEqn<N>::evalRhs( TVector <N> *v, uint i ) const {
   if (f)
      return f->eval(v);
   else{ 
      cerr<<"TCdrEqn<N>::evalRhs: no rhs function defined "<<endl;
      return 0;
   }
         
}



// get and set diffusion functions
template <int N>
TFunction<N> *
TCdrEqn<N>::getDiffFunctions(const int i, const int j) const {
   
   TFunction<N> * Pvf=NULL;
   if ((i< 0) || (i>2)) 
      cerr<<"TCdrEqn::getDiffFunction: argument i= "<<i<<" out of range"<<endl;
   else
      if ((j< 0) || (j>2)) 
         cerr<<"TCdrEqn::getDiffFunction: argument j= "<<j<<" out of range"<<endl;
      else
         Pvf=d[i][j];
   return Pvf;
} 
   

template <int N>
void 
TCdrEqn<N>::setDiffFunctions(  TFunction<N> * z, const int i, const  int j) {
   if ((i< 0) || (i>N+1)) 
      cerr<<"TCdrEqn::setDiffFunction: argument i= "<<i<<" out of range"<<endl;
   else
      if ((j< 0) || (j>N+1)) 
         cerr<<"TCdrEqn::setDiffFunction: argument j= "<<j<<" out of range"<<endl;
      else
         d[i][j]=z;
}



// get and set convection functions
template <int N>
TFunction<N> * 
TCdrEqn<N>::getConvFunctions(int i) const {
   TFunction<N> * Pvf=NULL;
   if ((i> -1) && (i<N)) 
      Pvf=b[i];
   else 
      cerr<<"TCdrEqn::getConvFunction: argument i= "<<i<<" out of range"<<endl;
   return Pvf;
}

template <int N>
void 
TCdrEqn<N>::setConvFunctions(  TFunction<N>* z, int i){
   if ((i> -1) && (i<N)) 
      b[i] = z;
   else cerr<<"TCdrEqn::setConvFunction: argument i= "<<i<<" out of range"<<endl;
}


template class TCdrEqn<2>;
template class TCdrEqn<3>;
