//
//  Project   : libnet
//  File      : TStokesBasisEqn3D.cc
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for pdes
//  Date      : 14.08.1997
//  Changes   : 
//
// arch-tag: 359fe126-dac7-4f35-9cd8-3d3fc0e84cab
//

#include "TStokesBasisEqn.hh"

// constructor	
template <int N>
TStokesBasisEqn<N>::TStokesBasisEqn ( TFloat aEpsilon, 
                                      TBlockVector<TFloat> *aConvVector)
        : TConvEqn<N>()
{ 
   epsilon = aEpsilon;
   convVector = aConvVector;
   for (uint i = 0; i < N; i++)
       b[i] = NULL;
}

//
// get and set convection direction
//
template <int N>
void
TStokesBasisEqn<N>::setConvVector ( TBlockVector<TFloat> *newConvVector ) 
{
    convVector = newConvVector; 
}

template <int N>
TBlockVector<TFloat>*
TStokesBasisEqn<N>::getConvVector ()
{
    return (convVector);
}

// delete convection Vector
template <int N>
void 
TStokesBasisEqn<N>::deleteConvVector () 
{
   delete (convVector);
   convVector = NULL;
}

// init convVector using the convection functions
template <int N>
void 
TStokesBasisEqn<N>::initConvVector ( uint size, 
                                     TDynArray< TVector<N>* >& coordVec )
{
      uint no[N+1];
      for (uint i = 0; i < N+1; i++) no[i] = size;
      
      // allocation of convVector (if necessary)
      if ( convVector == NULL )
         convVector = new TBlockVector<TFloat> ( N+1, no );
      
      // evaluate b  at the coordinates belonging to unknown 'k'
      // write result to convVector
      TVector<N> *b = new ( TVector<N> );
      
      for ( uint k = 0; k < size; k++ ){
         evalConv( coordVec[k], b);    
         for (uint i = 0; i < N; i++) 
            (*convVector)[i][k] = (*b)[i];
	  }
}
  

// get and set convection functions
template <int N>
TFunction<N> * 
TStokesBasisEqn<N>::getConvFunctions(int i) const 
{
    TFunction<N> * Pvf = NULL;
    if ((i > -1) && (i < N)) 
        Pvf = b[i];
    else 
        cerr << "TStokesBasisEqn::getConvFunction: argument i= " << i
             << " out of range" << endl;
    return Pvf;
}

template <int N>
void 
TStokesBasisEqn<N>::setConvFunctions(  TFunction<N>* z, int i){
   if ((i > -1) && (i < N)) 
      b[i] = z;
   else cerr << "TStokesBasisEqn::setConvFunction: argument i= " << i 
             << " out of range" << endl;
}

// some types
template class TStokesBasisEqn<2>;
template class TStokesBasisEqn<3>;
