//
//  Project   : libnet
//  File      : convDiffDoFManager.cc
//  Author    : Sabine Gutsch
//  Purpose   : class for DoF manager for piecewise linear
//              elements for the convection-diffusion equation
//
// arch-tag: 3e053cbc-016f-4d35-90c4-eea61bb11579
//

#include "convDiffDoFManager.hh"

// constructors	
T2DConfDiffDoFManager::T2DConfDiffDoFManager () {}

// destructor
T2DConfDiffDoFManager::~T2DConfDiffDoFManager () {}

TBlockMatrix* 
T2DConfDiffDoFManager::buildStiffnessMatrix( TNet<2> *cNet, TNet<2> *fNet ) 
{
	// to avoid compiler warning (fNet is not used)
	fNet = cNet;
	// create the blockmatrix
	// first the blockstructure of the matrix
	uint noOfDoFs = cNet->getNoOfInteriorNodes();
	// vertexName < BOUNDARY_NAME ---> vertex is interiour
	// vertexName > BOUNDARY_NAME ---> vertex is on boundary
	uint BOUNDARY_NAME;
	BOUNDARY_NAME = UINT_MAX - cNet->getNoOfBoundaryNodes();

	uint matrixSize[1][1] = { { noOfDoFs } };

	TBlockMatrix *blockMatrix = new TBlockMatrix( 1, (uint*) matrixSize );

	uint  i;
	uint vertexName, neighbourName;
	uint  *noOfEntries = new uint[noOfDoFs];
	memset( noOfEntries, 0, sizeof(uint)*noOfDoFs );
        
	const uint maxSize = 32;
	uint  newIndex[maxSize];
	TFloat newValue[maxSize];
	memset( newValue, 0, sizeof(TFloat)*maxSize );

	// build blockMatrix[0][0]
	T2DNet::TCellListIterator iter( cNet->getList() );
	for ( TCell<2> *cell = iter.first(); 
		  cell != NULL; 
		  cell = iter.next() )
		for (i = 0; i < 3; i++)
		{

			vertexName = cell->getNode(i)->getName();
			if (vertexName < BOUNDARY_NAME)
			{
				// build matrix data for vertexName 
				// if it does not exist yet
				if ( noOfEntries[vertexName] == 0 ) {
					(*blockMatrix)[0][0][vertexName].set(maxSize, newIndex, newValue);
					// set diagonal entry
					(*blockMatrix)[0][0][vertexName].setIndex(0, vertexName);
					noOfEntries[vertexName]++;
				}
				// add neighbours
				neighbourName = cell->getNode((i+1)%3)->getName();
				if ( (neighbourName < BOUNDARY_NAME) &&
					 ((*blockMatrix)[0][0][vertexName].hasIndex( neighbourName, noOfEntries[vertexName] ) == FALSE)) 
					(*blockMatrix)[0][0][vertexName].setIndex( noOfEntries[vertexName]++, neighbourName );
					
				neighbourName = cell->getNode((i+2)%3)->getName();
				if ( (neighbourName < BOUNDARY_NAME) &&
					 ((*blockMatrix)[0][0][vertexName].hasIndex( neighbourName, noOfEntries[vertexName] ) == FALSE)) 
					(*blockMatrix)[0][0][vertexName].setIndex( noOfEntries[vertexName]++, neighbourName );					
			}// if

		}

	// adjust sizes of matrixData
	for (i = 0; i < noOfDoFs; i++)
		(*blockMatrix)[0][0][i].set( noOfEntries[i] );

	delete[] noOfEntries;

	return (blockMatrix);
}
