//
//  Project     : libgraph
//  File        : T3DPlanarNode.cc
//  Author      : Ronald Kriemann
//  Purpose     : class for a node in a planar graph in 3D
//
// arch-tag: ca809867-ed33-4309-84aa-0db7aa710ab5
//

#include "T3DPlanarNode.hh"

// used flags
#define EDGE_SORTED  32

/////////////////////////////////////////////////
//
// constructor and destructor
//
    
T3DPlanarNode::T3DPlanarNode ( TMarkableNode & node,
                               TPlanarCycle * pMinCyc, 
                               TPlanarCycle * pMaxCyc, 
                               TFloat w )
        : TPlanarNode( node, pMinCyc, pMaxCyc, w )
{
    pVector = NULL;
}

T3DPlanarNode::T3DPlanarNode ( uint n, int m,
                               TVector<3> * pVec,
                               TPlanarCycle * pMinCyc, 
                               TPlanarCycle * pMaxCyc, 
                               TFloat w )
        : TPlanarNode( n, m, pMinCyc, pMaxCyc, w )
{
    pVector = pVec;
}

T3DPlanarNode::~T3DPlanarNode ()
{
}

/////////////////////////////////////////////////
//
// access member variables/status
//
    
TVector<3> *
T3DPlanarNode::vector ()
{
    return pVector;
}

void
T3DPlanarNode::setVector ( TVector<3> * pVec )
{
    pVector = pVec;
}

//
// check/set if edges are sorted
//
const bool
T3DPlanarNode::isSorted () const
{
    return hasFlag( EDGE_SORTED );
}

void
T3DPlanarNode::setSorted ( const bool & b )
{
    if (b == TRUE)
        setFlag( EDGE_SORTED );
    else
        clearFlag( EDGE_SORTED );
}    

/////////////////////////////////////////////////
//
// virtual constructors
//
    
T3DPlanarNode *
T3DPlanarNode::clone ()
{
    return new T3DPlanarNode( name, _mark,
                              pVector,
                              pMinCycle, pMaxCycle,
                              _weight );
}

T3DPlanarNode *
T3DPlanarNode::create ( uint n )
{
    return new T3DPlanarNode( n );
}
