//
//  Project     : libgraph
//  File        : TMatrixGraph.cc
//  Author      : Ronald Kriemann
//  Purpose     : class to build graph of a matrix
//
// arch-tag: 2f6a14b0-760f-420b-a99a-9db241f23c7e
//

#include "criterion.hh"
#include "nodes.hh"

#include "TMatrixGraph.hh"

////////////////////////////////////////
//
// constructor and destructor
//

TMatrixGraph::TMatrixGraph ( TMatrixBlock * pMatrix,
                             uint           criterion,
                             TFloat         xi )
{
    TMatrixGraph::pMatrix   = pMatrix;
    TMatrixGraph::criterion = criterion;
    TMatrixGraph::xi        = xi;

    buildGraph();
}

TMatrixGraph::~TMatrixGraph ()
{
}

////////////////////////////////////////
//
// encapsulation of graph creation
//

//
// actually build graph
//
void
TMatrixGraph::buildGraph ()
{
    if (pMatrix == NULL)
    {
        cout << "TMatrixGraph::buildGraph (...) : got NULL arg" << endl;
        return;
    }// if
    
    uint                 i, j;
    uint                 size, noOfEntries;
    uint               * entries, diagIdx;
    TDynArray< TNode * >  arrNodes;
    TNode               * pNode, * pAdjNode;
    TEdge               * pEdge;

    size = pMatrix->getNoOfRows();

    arrNodes.setSize( size );
    
    for ( i = 0; i < size; i++ )
    {
        pNode = new TMarkableNode( i );

        add( * pNode );
        arrNodes[i] = pNode;
    }// for

    for ( i = 0; i < size; i++ )
    {
        noOfEntries = (*pMatrix)[i].getNoOfEntries();
        entries = (*pMatrix)[i].getIndex();
        diagIdx = entries[0];

        pNode = arrNodes[diagIdx];

        for ( j = 1; j < noOfEntries; j++ )
        {
            pAdjNode = arrNodes[ entries[j] ];
            
            if ( isStrong( pNode, pAdjNode ) )
            {
                pEdge = new TEdge( * pNode, * pAdjNode );

                pNode->addOutEdge( * pEdge );
                pAdjNode->addInEdge( * pEdge );
            }// if
        }// for
    }// for
}

//
// decide whether an edge is strong or not
//
const bool
TMatrixGraph::isStrong ( TNode * pNode0, TNode * pNode1 ) const
{
    if ((pNode0 == NULL) || (pNode1 == NULL))
    {
        cout << "TMatrixGraph::isStrong (...) : got NULL arg" << endl;
        return FALSE;
    }// if

    uint   nodeName0, nodeName1;
    TFloat  diagValue, conValue0, conValue1;

    nodeName0 = pNode0->getName();
    nodeName1 = pNode1->getName();

    if (criterion == E3)
    {
        diagValue = (*pMatrix)[nodeName0][nodeName0];
        conValue0 = (*pMatrix)[nodeName0][nodeName1];
        
        if (fabs(conValue0) > fabs(xi * diagValue))
            return TRUE;
        else
            return FALSE;
    }// if

    if (criterion == E7)
    {
        conValue0 = (*pMatrix)[nodeName0][nodeName1];
        conValue1 = (*pMatrix)[nodeName1][nodeName0];
    
        if (fabs(conValue0) > fabs(xi * conValue1))
            return TRUE;
        else
            return FALSE;
    }// if

    cout << "TMatrixGraph::isStrong (...) : criterion not supported" << endl;
    exit( 1 );
    
    return FALSE;
}
