//
//  Project     : libgraph
//  File        : TRCMGraph.cc
//  Author      : Ronald Kriemann
//  Purpose     : class for a graph suitable for rcm
//
// arch-tag: 7d53139b-bd07-4f3f-8086-3feefc100422
//

#include "dynarray.hh"

#include "matrixblock.hh"

#include "nodes.hh"

#include "TRCMGraph.hh"

///////////////////////////////////////////////////
//
// class for RCM ordering
//

//
// constructor
//
TRCMGraph::TRCMGraph ( TMatrixBlock * pMatrix )
    : TGraph ()
{
    uint  i;
    uint  noOfRows = pMatrix->getNoOfRows();

    //
    // create all nodes
    //

    TDynArray< TMarkableNode * >  arrNodes( noOfRows, 0 );
    TMarkableNode                *pNode;

    for (i = 0; i < noOfRows; i++)
    {
        pNode = new TMarkableNode( i );
        arrNodes[i] = pNode;
        add( *pNode );
    }// for

    //
    // create edges between nodes
    // (graph is undirected -> only incident edges will be build)
    //

    TListIterator<TNode> iterNodes( listNodes );

    TEdge  *pEdge;
    uint   nodeName;
    uint  *arrIndex;
    uint    noOfEntries;

    iterNodes.first();
    while ((pNode = (TMarkableNode*) iterNodes++) != NULL)
    {
        nodeName = pNode->getName();

        arrIndex    = (*pMatrix)[nodeName].getIndex();
        noOfEntries = (*pMatrix)[nodeName].getNoOfEntries();

        for (i = 0; i < noOfEntries; i++)
            if (arrIndex[i] != nodeName)
            {
                pEdge = new TEdge( *pNode, *(arrNodes[arrIndex[i]]) );
                //				pNode->addInEdge( *pEdge );
                arrNodes[arrIndex[i]]->addInEdge( *pEdge );
            }// if
    }// while
}
