//
//  Project     : libgraph
//  File        : T3DCellEdgeGraph.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a graph consisting of celledges
//
// arch-tag: 7ede96f3-8eea-43fd-b1ad-8279b8b5d78c
//

#ifndef __T3DCELLEDGEGRAPH_HH
#define __T3DCELLEDGEGRAPH_HH

#include "basegraph.hh"

// forward decl.
template <int N> class TConvEqn;
template <int N> class TNet;
template <int N> class TVector;
class T3DCellEdge;
class T3DFrontier;
class TMatrixBlock;
class TFSNode;

//
//
//
class T3DCellEdgeGraph : public TGraph
{
protected:
    // parameters for choosing strong edges
    TFloat         kappa, sigma;

    // upper bound for minimal angles (phi,b)
    TFloat         gamma;

    // choose strong edges by matrixentries
    // (will be used if non-NULL)
    TMatrixBlock * pMatrixBlock;

    // pointer to node in metagraph
    TFSNode      * pFSNode;
    
public:
    //////////////////////////////////////////////////
    //
    // constructors and destructors
    //

    // constructor
    T3DCellEdgeGraph ();
    T3DCellEdgeGraph ( TNet<3> * pNet, TConvEqn<3> * pEqn,
                       TFloat kappa, TFloat gamma );
    T3DCellEdgeGraph ( TNet<3> * pNet, TConvEqn<3> * pEqn,
                       TMatrixBlock * pMatrix,
                       TFloat kappa, TFloat sigma, TFloat gamma );

    // destructor
    virtual ~T3DCellEdgeGraph ();

    //////////////////////////////////////////////////
    //
    // virtual constructors
    //

	// create new graph
	virtual T3DCellEdgeGraph * clone ();

    //////////////////////////////////////////////////
    //
    // misc. stuff
    //

    // access node in metagraph
    virtual TFSNode * fsNode () const;
    virtual void setFSNode ( TFSNode * pNode );
    
protected:
    //////////////////////////////////////////////////
    //
    // encapsulate creation of graph
    //
    
    virtual void buildGraph ( TNet<3> * pNet, TConvEqn<3> * pEqn );
    
    virtual bool isStrongEdge ( T3DCellEdge * pEdge,
                                TConvEqn<3> * pEqn,
                                TVector<3> * pVecConvDir ) const;

    virtual bool isBoundaryFace ( T3DFrontier * pFace ) const;

    // check if face is used
    virtual bool isFaceUsed ( T3DFrontier * pFace ) const;
    
    // mark face as used
    virtual void setFaceUsed ( T3DFrontier * pFace ) const;
};

#endif  // __T3DCELLEDGEGRAPH_HH
