//
//  Project     : libgraph
//  File        : T3DFaceOrient.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a implementing methods to
//                handle orientation of 3d faces
//
// arch-tag: db6bb9d8-c225-4ea1-9ddf-94b17ae1c3cb
//

#ifndef __T3DFACEORIENT_HH
#define __T3DFACEORIENT_HH

// forward decl.
class T3DFrontier;
template <int N> class TVector;

//
// - class for a implementing methods to
//   handle orientation of 3d faces
// - only implemented for triangles
// - orientation is stored in data-variable of face
//   as indices of face-nodes
// - all methods must be static
//
class T3DFaceOrient
{
public:
    /////////////////////////////////////////////
    //
    // constructor
    //
    T3DFaceOrient ();

    /////////////////////////////////////////////
    //
    // methods to handle orientation
    //

    // set orientation of face (explicit/by adjacent face)
    static void setOrient ( T3DFrontier * pFace,
                            uint i0, uint i1, uint i2 );
    static void setOrient ( T3DFrontier * pFace,
                            T3DFrontier * pOrientFace );

    // check if (v0,v1) is in orientation of face
    static bool cmpOrient ( T3DFrontier * pFace,
                            TVector<3>  * pNode0,
                            TVector<3>  * pNode1 );

    // check if orientation of (adjacent) faces is compatible
    static bool chkOrient ( T3DFrontier * pFace1,
                            T3DFrontier * pFace2 );

    // check if face has legal orientation
    static bool hasOrient ( T3DFrontier * pFace );

    // get/set each index sep.
    static uint getOrientIdx ( T3DFrontier * pFace,
                               uint idx );
    static void setOrientIdx ( T3DFrontier * pFace,
                               uint idx, uint i );

    // print orientation
    static void showOrient ( T3DFrontier * pFace );
};

#endif  // __T3DFACEORIENT_HH
