//
//  Project     : libgraph
//  File        : T3DGraph.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a graph in 3D
//
// arch-tag: c7006811-59e5-4e44-a055-e69de40b6d2a
//

#ifndef __T3DGRAPH_HH
#define __T3DGRAPH_HH

#include "basegraph.hh"

// forward decl.
template <int N> class TNet;
class TMatrixBlock;
class T3DCellEdge;
class T3DCellEdgeGraph;
class TNode;

//
// this class builds a graph of nodes in 3D out of
// the given net and matrix
//
class T3DGraph : public TGraph
{
protected:
    // criterion and parameter to decide about
    // strongness of edges
    uint            criterion;
    TFloat          xi;
    
    // matrix to decide of edges are strong
    TMatrixBlock  * pMatrix;
    
public:
    ///////////////////////////////////////////
    //
    // constructor and destructor
    //

    T3DGraph ();

    T3DGraph ( TNet<3> * pNet, TMatrixBlock * pMatrix,
               uint criterion, TFloat xi );

    T3DGraph ( TNet<3> * pNet,
               T3DCellEdgeGraph * pEdgeGraph,
               TMatrixBlock * pMatrix,
               uint criterion, TFloat xi );

    ~T3DGraph ();

    //////////////////////////////////////////////////
    //
    // virtual constructors
    //

	// create new graph
	virtual T3DGraph * clone ();

protected:
    ///////////////////////////////////////////
    //
    // encapsulate graph building
    //

    // build graph
    void buildGraph ( TNet<3> * pNet );
    void buildGraph ( TNet<3> * pNet,
                      T3DCellEdgeGraph * pEdgeGraph );

    // check if given celledge lies (partially) on a boundary
    bool onBoundary ( T3DCellEdge * pEdge ) const;

    // look if edge from node0 to node1 is a strong edge
    bool isStrong ( TNode * pNode0, TNode * pNode1 ) const;
    
    // create an edge from node0 to node1
    void buildEdge ( TNode * pNode0, TNode * pNode1 );

    // return array with geometry info about all nodes
    TDynArray< TVector<3> * > * getCoord ( TNet<3> * pNet );
};

#endif  // __T3DGRAPH_HH
