//
//  Project :  libvisual
//  File    :  T3DGraphView.hh
//  Purpose :  class for visualization of a 3D graph
//  Author  :  Ronald Kriemann
//
// arch-tag: 48443978-fff4-411d-93c4-e556192ba251
//

#ifndef __T3DGRAPHVIEW_HH
#define __T3DGRAPHVIEW_HH

#include "view.hh"

// forward decl
class T3DCellEdgeGraph;
class T3DSurfaceGraph;
class T3DGraph;
class TPermutation;

//
//
//
class T3DGraphView : public TView
{
protected:
    // the graphs to be drawn
    T3DCellEdgeGraph  * pCellEdgeGraph;
    T3DSurfaceGraph   * pSurfaceGraph;
    T3DGraph          * pNetGraph;

    // set of flowsurfaces
    TList< T3DCellEdgeGraph >  * pFSList;

    // set of surfacegraphs
    TList< T3DSurfaceGraph >   * pSGList;
    
    // set of cycles
    TList< TGraph >            * pListCycles;
    
    // permutation to use for nodenames
    TPermutation      * pPermutation;
    
public:
    ///////////////////////////////////////////
    //
    // constructors and destructors
    //

    T3DGraphView ( T3DCellEdgeGraph * pGraph );
    T3DGraphView ( T3DSurfaceGraph * pGraph );
    T3DGraphView ( T3DGraph * pGraph );
    T3DGraphView ( TList< T3DCellEdgeGraph > * fsList );
    T3DGraphView ( TList< T3DSurfaceGraph >  * sgList );
    T3DGraphView ( TList< TGraph > * listCycles );

    ~T3DGraphView ();

    ///////////////////////////////////////////
    //
    // access member variables
    //

    void setPermutation ( TPermutation * pPerm );
    
    ///////////////////////////////////////////
    //
    // visualization methods
    //

    virtual void draw3D ( T3DPrinter & p );

protected:
    // reset all member variables
    void reset ();
    
    //
    // encapsulate drawing for each graphtype
    //
    void drawCellEdgeGraph ( T3DPrinter & p );
    void drawSurfaceGraph  ( T3DPrinter & p );
    void drawNetGraph      ( T3DPrinter & p );
    void drawFlowSurfaces  ( T3DPrinter & p );
    void drawSurfaceGraphs ( T3DPrinter & p );
    void drawPlanarCycles  ( T3DPrinter & p );
};

#endif  // __T3DGRAPHVIEW_HH
