//
//  Project     : libgraph
//  File        : T3DPlanarNode.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a node in a planar graph in 3D
//
// arch-tag: bee6ac43-895b-4e18-8bee-5dc5461003c4
//

#ifndef __T3DPLANARNODE_HH
#define __T3DPLANARNODE_HH

#include "vector.hh"

#include "TPlanarNode.hh"

//
//
//
class T3DPlanarNode : public TPlanarNode
{
protected:
    // vector assoc. with the node
    TVector<3>  * pVector;

public:
    /////////////////////////////////////////////////
    //
	// constructor and destructor
    //
    
	T3DPlanarNode ( TMarkableNode & node,
                    TPlanarCycle * pMinCyc = NULL, 
                    TPlanarCycle * pMaxCyc = NULL, 
                    TFloat w = 0.0 );
	T3DPlanarNode ( uint n, int m = NO_MARK,
                    TVector<3> * pVec = NULL,
                    TPlanarCycle * pMinCyc = NULL, 
                    TPlanarCycle * pMaxCyc = NULL, 
                    TFloat w = 0.0 );

	virtual ~T3DPlanarNode ();

    /////////////////////////////////////////////////
    //
    // access member variables/status
    //
    
	virtual TVector<3> * vector ();
	virtual void setVector ( TVector<3> * pVec );

    // check/set if edges are sorted
    virtual const bool isSorted () const;
    virtual void setSorted ( const bool & b );
    
    /////////////////////////////////////////////////
    //
    // virtual constructors
    //
    
	virtual T3DPlanarNode * clone ();
	virtual T3DPlanarNode * create ( uint n );
};

#endif  // __T3DPLANARNODE_HH
