//
//  Project   : libnet
//  File      : T3DTriangle.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for a triangle in 3d
//
// arch-tag: 7a9381b2-7911-4aa0-91f9-82ae22d7fcf0
//

#ifndef __T3DTRIANGLE_HH
#define __T3DTRIANGLE_HH

#include "T3DFrontier.hh"

//
//
//
class T3DTriangle : public T3DFrontier
{
public:
    // constructor
    T3DTriangle ( T3DFrontier *parent = NULL,
                  TBoundary *boundary = INTERIOR );
    T3DTriangle ( T3DFrontier *parent,
                  T3DCellEdge *e0, 
                  T3DCellEdge *e1, 
                  T3DCellEdge *e2, 
                  TBoundary *boundary = INTERIOR );
    T3DTriangle ( T3DFrontier *parent,
                  T3DCellEdge *e0, 
                  T3DCellEdge *e1, 
                  T3DCellEdge *e2,
                  const bool dir0,
                  const bool dir1,
                  const bool dir2,
                  TBoundary *boundary = INTERIOR );

    // set direction of each edge for continous access
    virtual void initEdgeDir ();

    // return info about built midnodes
    virtual const bool hasMidNode () const;

    // make the midnode(s) of the frontier
    virtual void buildMidNode ();

    // build the child(ren)
    virtual void buildChild ();

    // return i'th child
    virtual TFrontier<3>* getChild ( const TCell<3>* cell, uint i ) const;
};

#endif  // __T3DTRIANGLE.HH
