//
//  Project   : libnet
//  File      : basisHierarchy.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for creating a hierarchy of 
//              stiffness matrices and prolongations 
//
// arch-tag: 34dd0700-3397-4648-b06d-ea7fe5986943
//

#ifndef __TBASISHIERARCHY_HH
#define __TBASISHIERARCHY_HH

#include "memory.hh"

#include "vector.hh"
#include "blockvector.hh"
#include "blockmatrix.hh"

#include "nethierarchy.hh"

#include "TBasisDoFManager.hh"
#include "TBasisFE.hh"

//
// basis class for a finite element 
//

template <int N>
class TBasisHierarchy : public TMemory {

public:
    // constructor
    TBasisHierarchy() {}	
    // destructor
    virtual ~TBasisHierarchy() {}	
    // set up hierarchy
    virtual void buildHierarchy ( TBasisDoFManager<N> *dofManager,
                                  TBasisEqn<N> *eqn,
                                  TBasisFE<N> *fe,
                                  TNetHierarchy<N> *netHier, 
                                  TBlockMatrix **pMatrix, 
                                  TBlockVector<TFloat> **pRHS, 
                                  TBlockVector<TFloat> **pX ) = 0;
};

#endif  // __TBASISHIERARCHY_HH
