//
//  Project   : libnet
//  File      : TBoundaryDesc.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for representing info about boundaryvalues
//
// arch-tag: 3fd2e75e-f082-4e63-a57a-ddb5635717bc
//

#ifndef __TBOUNDARYDESC_HH
#define __TBOUNDARYDESC_HH

#include "memory.hh"
#include "multiref.hh"
#include "dynarray.hh"
#include "TFlagable.hh"

// default type
#define NO_TYPE  -1

// type for Dirichlet Nodes
#define DIR_TYPE  1

// type for Neumann Nodes
#define NEU_TYPE  2

//
//
//
class TBoundaryDesc : public TMemory, public TMultiReference, public TFlagable<uchar>
{
protected:
    // holds data about type in each component
    TDynArray<int>  arrType;

public:
    // constructor
    TBoundaryDesc ( int n = 0 );

    // destructor
    ~TBoundaryDesc ();

    // set (get) number of types to use
    uint getNoOfTypes () const;
    void setNoOfTypes ( uint n );
    
    // access type info
    const int getType ( uint i ) const;
    void setType ( uint i, int t );

    // access DoF info
    const bool isDoF ( uint i ) const;
    void setDoFInfo ( uint i, bool b );

    // automatic DoF info setting:
    //    DIR_TYPE: no DoF;  NEU_TYPE: DoF
    void setDoFInfo ();

    // forward decleration for TCdrBoundaryDesc
    virtual int getN() const;

};

#endif  // __TBOUNDARYDESC_HH
