//
//  Project   : libnet
//  File      : TCdrEqn.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for pdes
//
// arch-tag: a4d7e553-2002-4d08-a3bf-830a87334f94
//

#ifndef __TCDREQN_HH
#define __TCDREQN_HH

#include "TBasisCdrEqn.hh"

//
// basis class for a pde
//
template <int N>
class TCdrEqn : public TBasisCdrEqn<N> {

protected:
 
   // diffusion matrix
   TFunction<N>*  d[N][N];
  
   // convection vector
   TFunction<N>*  b[N];

   
   // reaction  function
   TFunction<N>* c;
   
   // rhs  function
   TFunction<N>*  f;
   
   
public:
   // constructors	
   TCdrEqn( TFloat aEpsilon = 1.0, 
            TBlockVector<TFloat> *aConvVector = NULL ); 
   
   // destructor
   virtual ~TCdrEqn () { }
  
  
   // evaluate Diffusion matrix
   virtual  void evalDiff(TVector <N> *v, TMatrix<N>  *D) const;
   
   // evaluate convection vector
   virtual  void evalConv(TVector <N> *v, TVector<N>  *B) const;
   
   // evaluate reaction 
   virtual TFloat evalReac(TVector <N> *v) const;
   
   // evaluate rhs
   virtual TFloat evalRhs(TVector <N> *v, uint i = 0 ) const;
 
  
   // get and set diffusion functions
   TFunction <N> * getDiffFunctions(const int i, const int j) const ;
   void setDiffFunctions( TFunction <N> * z, int i, int j);

   // get and set convection functions
   TFunction <N> * getConvFunctions(const int i) const ; 
   void setConvFunctions( TFunction <N> * z, int i);

   // get and set  reaction function
   TFunction <N> * getReac() const  {  return (c); }
   void setReacFunction( TFunction <N> * z) { c = z; }

   // get and set  rhs function
   TFunction <N> * getRhs() const  {  return (f); }
   void setRhsFunction( TFunction <N> *  z) { f = z; }
  
};

// some types

#endif //__TCDREQN_HH
