//
//  Project   : libnet
//  File      : TConvDiffDoFManager.hh
//  Author    : Sabine Gutsch /tpr
//  Purpose   : class for DoF manager for piecewise linear
//              elements for the convection-diffusion equation
//
// arch-tag: 73b798c6-b5b0-4167-a953-78e73db659b7
//

#ifndef __TCONVDIFFDOFMANAGER_HH
#define __TCONVDIFFDOFMANAGER_HH

#include "TBasisDoFManager.hh"

//
// class for DoF manager for piecewise linear
// elements for the convection-diffusion equation
//
template <int N> 
class TConvDiffDoFManager : public TBasisDoFManager<N> 
{
protected:

    uint noOfDoFs;
    uint noOfInnerDoFs;
    // global counter for DoFs on boundary (for each net)
    uint noOfBoundaryDoFs;
    
public:
    // constructors	
    TConvDiffDoFManager ();
    // destructor
    ~TConvDiffDoFManager ();

    // get noOfDofs;  
    uint getNoOfDoFs() const {return noOfDoFs;}
   
    // initialise Dofs
    virtual void initDoFDesc( TNet<N> *cNet, TNet<N> *fNet = NULL );
  
	 // build stiffness matrix
    TBlockMatrix* buildStiffnessMatrix( TNet<N> *cNet, TNet<N> *fNet = NULL );
};

#endif  // __TCONVDIFFDOFMANAGER_HH
