//
//  Project   : libnet
//  File      : cyclicReduction.hh
//  Author    : tpr
//  Purpose   : cyclic reduction solver 
//              (cf. A. Reusken, IGPM-Berich 144, Okt. 1997, RWTH Aachen)
//
// arch-tag: f3101912-1070-4736-9a10-fcba6e295b07
//

#ifndef __CYCLICREDUCTION_HH
#define __CYCLICREDUCTION_HH

#include "solver.hh"

//
// class containing all the CyclicReduction parameters
//

class TCyclicReductionInfo {

protected:
	// lowest and highest level
   uint    low, high, noOfLevel;
   TSolver  *pInnerSolver, *pCoarseSolver;

	// 1: V-Zyklus, 2: W-Zyklus
   uint    recursiveCalls;

	// coarse grid correction damping
   TFloat  CGCdamping;

public:
   // constructor
   TCyclicReductionInfo ( uint alow, uint ahigh, uint aRecCall,
                          TSolver* apInnerSolver, 
                          TSolver* apCoarseSolver );
  
   // get lowest/highest level, noOfLevel
   uint getHighLevel ();
   uint getLowLevel ();
   uint getNoOfLevel ();
  
   // set InnerSolver		
   void setInnerSolver ( TSolver* apInnerSolver );
   
   // set coarse grid solver
   void setCoarseSolver ( TSolver* apCoarseSolver );
};


//
// class for a CyclicReduction solver
// 
class TCyclicReduction : public TSolver, public TCyclicReductionInfo {

protected:
   TBlockVector<TFloat> d;

public:
   // constructor
   TCyclicReduction ( TBlockMatrix *apA, TBlockVector<TFloat> *apRHS, 
                      TBlockVector<TFloat> *apX, 
                      uint alow, uint ahigh, uint aRecCall,  
                      TSolver* apInnerSolver,
                      TSolver* apCoarseSolver, 
                      uint amaxiter = DEFAULT_MAX_ITERATION,
                      TFloat aaccur = DEFAULT_ACCURACY,
                      uint averb = 0 );

   // one CyclicReduction step on level 'level'
   void step ( uint level );
  
   // one CyclicReduction step on highest level
   int step ();
  
   // get matrix, rhs and x  resp.
   TBlockMatrix *getMatrix() { return (pA); }
   TBlockVector<TFloat> *getRHS() { return (pRHS); }
   TBlockVector<TFloat> *getX() { return (pX); }
  
   // update pX on all levels by restricting fine grid pX to
   // coarser levels
   //void updateXHierarchy();
  
   // add/subtract multiple of identity to all levels
   //void addIdentity( const TFloat factor );
   //void subtractIdentity( const TFloat factor );
};

   

#endif  // __CYCLICREDUCTION_HH
