//
//  Project   : libnet
//  File      : TDomain.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for representing a domain in R^n
//
// arch-tag: e7036993-5088-43cb-9a3e-1201ff753bb6
//

#ifndef __TDOMAIN_HH
#define __TDOMAIN_HH

#include "memory.hh"
#include "list.hh"

#include "vector.hh"

#include "TBoundary.hh"

//
// - this class represents a domain by its boundaries (or
//   the characteristic function)
// - for future : automatic mesh-generation
//
class TDomain : public TMemory
{
protected:
    // minimal (lower left) and maximal (upper right) point of domain
    // (this is only for more optimal access of char. function)
    TVector<3>          vecMin, vecMax;

    // list of boundaries representing the domain
    TList< TBoundary >  listBound;

public:
    ////////////////////////////////////////////////////////////
    //
    // constructor and destructor
    //

    TDomain ();

    virtual ~TDomain ();

    ///////////////////////////////////////////////////////////
    //
    // access member variables
    //

    virtual TVector<3> * min ();
    virtual TVector<3> * max ();

    virtual void setMin ( TVector<3> * pVector );
    virtual void setMax ( TVector<3> * pVector );

    virtual const TList< TBoundary > & getBoundaries () const;

    ///////////////////////////////////////////////////////////
    //
    // boundary management
    //
    
    // add a boundary to list
    virtual void addBoundary ( TBoundary * pBoundary );

    // return boundary (or INTERIOR) assoc. to given point
    virtual TBoundary * boundary ( TFloat x, TFloat y, TFloat z );
    virtual TBoundary * boundary ( TVector<3> * v );
    
    ///////////////////////////////////////////////////////////
    //
    // misc methods
    //

    // characteristic function of domain
    virtual bool charFunc ( TVector<3> * pVector ) const;
};

#endif  // __TDOMAIN_HH
