//
//  Project   : libmisc
//  File      : TFlagable.hh
//  Author    : Ronald Kriemann
//  Purpose   : class providing ability to store and access flags
//
// arch-tag: 1aae00b3-f017-4e4f-94e3-fbf81ee63f65
//

#ifndef __TFLAGABLE_HH
#define __TFLAGABLE_HH

#include "defines.hh"

// forward friend declarations
template <class T> class TFlagable;
template <class T> ostream & operator << ( ostream &, const TFlagable<T> & );

//
// class providing ability to store and access flags
// - flags are stored as bits
// - template is used to set storagesize
// - preferred types : uchar, ushort, uint and uint
//
template <class T>
class TFlagable
{
protected:
    // stored flags (bitcoded)
    T    _flags;

public:
    ////////////////////////////////////////
    //
    // constructor and destructor
    //

    TFlagable ();
    TFlagable ( const T & mask );
    TFlagable ( const TFlagable<T> & flag );

    virtual ~TFlagable ();

    ////////////////////////////////////////
    //
    // access member variable
    //

    virtual const bool flag ( const T & mask ) const;
    virtual const bool hasFlag ( const T & mask ) const;
    
    virtual const T & flags () const;

    virtual void setFlag ( const T & mask );

    virtual void clearFlag ( const T & mask );
    virtual void clearFlags ();

    ////////////////////////////////////////
    //
    // misc operators
    //

    TFlagable & operator = ( const TFlagable<T> & flag );

    friend ostream & operator << <> ( ostream & os, const TFlagable<T> & flag );
};

#endif  // __TFLAGABLE_HH
