//
//  Project   : libnet
//  File      : TGMRES.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for solver implementing GMRES
//
// arch-tag: a60c8725-5abc-4a05-82a3-6a3494ecd09e
//

#ifndef __TGMRES_HH
#define __TGMRES_HH

#include "dynarray.hh"
#include "blockvector.hh"

#include "TPrecSolver.hh"

//
//
//
class TGMRES : public TPrecSolver
{
protected:
    // number of steps before restart
    int       _m;

    // array with old directions
    TDynArray< TBlockVector< TFloat > >  _v;
    
public:
    //
    // constructor and destructor
    //
    TGMRES( TSolver  * prec     = NULL,
            uint       amaxiter = DEFAULT_MAX_ITERATION, 
            TFloat     aaccur   = DEFAULT_ACCURACY, 
            uint       averb    = 0 );

    ~TGMRES ();
    
    //
    // iteration
    //
    
    void init ();
    int  step ();
    void solve ();
    void solve ( int                     level,
                 TBlockMatrix         ** A,
                 TBlockVector<TFloat> ** RHS,
                 TBlockVector<TFloat> ** x );

    //
    // parameter settings
    //

    void setRestart ( int m );
};

#endif  // __TGMRES_HH
