//
//  Project     : libgraph
//  File        : TPlanarNode.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a node in a planar graph
//
// arch-tag: 4e8dbfc0-cbc5-4f8b-a7be-080de066d260
//

#ifndef __TPLANARNODE_HH
#define __TPLANARNODE_HH

#include "list.hh"
#include "TFlagable.hh"

#include "nodes.hh"

// forward decl.
class TPlanarCycle;

//
// baseclass for all algorithms based on minimal
// and maximal cycles in planar graphs
//
class TPlanarNode : public TMarkableNode, public TFlagable<uchar>
{
protected:
	// store min/max cycle we are on
	TPlanarCycle    * pMinCycle;
    TPlanarCycle    * pMaxCycle;

	// weight of the node
	TFloat            _weight;

	// add. mark for max. cycle (old mark is for min cycle)
	int               _maxMark;

	// this list is for remembering nodes which are
	// adj. by weak edges
	TList< TNode >  * pListBlock;

public:
    /////////////////////////////////////////////////
    //
	// constructor and destructor
    //
    
	TPlanarNode ( TMarkableNode & node,
                  TPlanarCycle * pMinCyc = NULL, 
                  TPlanarCycle * pMaxCyc = NULL, 
                  TFloat w = 0.0 );
	TPlanarNode ( uint n, int m = NO_MARK,
                  TPlanarCycle * pMinCyc = NULL, 
                  TPlanarCycle * pMaxCyc = NULL, 
                  TFloat w = 0.0 );

	virtual ~TPlanarNode ();

    /////////////////////////////////////////////////
    //
    // access member variables
    //
    
	virtual TPlanarCycle * cycle ( uint type );
	virtual void setCycle ( TPlanarCycle * pCycle, uint type );

	virtual const int mark ( uint type = 0 ) const;
	virtual void setMark ( int m, uint type = 0 );

	virtual const TFloat weight () const;
	virtual void setWeight ( TFloat w );

	virtual const bool isVisited () const;
	virtual void setVisited ( bool v );

	virtual const bool isDeleted ( uint type ) const;
	virtual void setDeleted ( bool d, uint type );

	virtual const bool isInStack () const;
	virtual void setInStack ( bool b );

    virtual TPlanarNode * left ( bool bCheckSucc = FALSE ) const;
    virtual TPlanarNode * right ( bool bCheckCycle = FALSE ) const;

    /////////////////////////////////////////////////
    //
    // methods to manage list of weakly connected nodes
    //
    
	virtual TList<TNode> * blockList () const;

	virtual void addLeftToBlockList  ( TNode * pNode );
	virtual void addRightToBlockList ( TNode * pNode );
	
    /////////////////////////////////////////////////
    //
    // virtual constructors
    //
    
	virtual TPlanarNode * clone ();
	virtual TPlanarNode * create ( uint n );
};

#endif  // __TPLANARNODE_HH
