//
//  Project   : libnet
//  File      : TPrecSolver.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : baseclass for solver-classes using a preconditioner
//
// arch-tag: e79043e2-5d94-4dcf-b0f2-7a3b20b5d7d4
//

#ifndef __TPRECSOLVER_HH
#define __TPRECSOLVER_HH

#include "solver.hh"
#include "secondNF.hh"

//
//
//
class TPrecSolver : public TSolver
{
protected:
	// preconditioner
    TSolver    * pPrecond;

    // is precond in 2nd normal form
    bool         bSecondNF;
    
public:
	// constructor
	TPrecSolver( TSolver              * prec    = NULL,
                 TBlockMatrix         * pA      = NULL, 
                 TBlockVector<TFloat> * pRHS    = NULL, 
                 TBlockVector<TFloat> * pX      = NULL, 
                 uint                   maxiter = DEFAULT_MAX_ITERATION, 
                 TFloat                 accur   = DEFAULT_ACCURACY, 
                 uint                   verb    = 0 );

    //
    // access precond.
	//

    virtual TSolver * getPrec () const;
	virtual void setPrec ( TSolver * precond );

    virtual const bool isPrecSecondNF () const;

    //
    // overload naming
    //

    virtual void cleanName ();
    virtual void setName ( const TString & n );
};

#endif  // __TPRECSOLVER_HH
