//
//  Project   : libnet
//  File      : TStokesBasisEqn.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for Navier-Stokes Equations
//
// arch-tag: 4c29a89f-9949-42c9-aa47-cdfd39ac7a0f
//

#ifndef __TSTOKESBASISEQN_HH
#define __TSTOKESBASISEQN_HH

#include "memory.hh"
#include "parray.hh"
#include "vertex.hh"

#include "vector.hh"
#include "TMatrix.hh"
#include "blockvector.hh"

#include "TConvEqn.hh"


//
// basis class for a pde
//
template <int N>
class TStokesBasisEqn : public TConvEqn<N>
{
protected: 
    TFloat epsilon;

    // (previous) flow direction   
    TBlockVector<TFloat> *convVector;

    // convection functions
    TFunction<N>*  b[N];
   
    
public:
    // constructors	
    TStokesBasisEqn ( TFloat aEpsilon = 1.0, 
                      TBlockVector<TFloat> *aConvVector = NULL );
    // destructor
    virtual ~TStokesBasisEqn () { }
   
    // get and set convection direction
    void setConvVector ( TBlockVector<TFloat> *newConvVector );
    TBlockVector<TFloat>* getConvVector ();
      
    // get and set convection functions
    TFunction <N> * getConvFunctions(const int i) const ; 
    void setConvFunctions( TFunction <N> * z, int i);

    // init convVector using the convection functions
    virtual void initConvVector ( uint size, 
                                  TDynArray< TVector <N> * > & coordVec );

    // delete convection Vector
    void deleteConvVector ();
   
    // get and set epsilon
    TFloat getEpsilon( ) { return (epsilon); }
    void setEpsilon ( TFloat aEps ) { epsilon = aEps; }
      
   // evaluate reaction 
   virtual TFloat evalReac(TVector <N> *v) const = 0;

    // evaluate rhs
    virtual TFloat evalRhs( TVector<N> *v, uint i ) const = 0;

    // evaluate convection vector
    virtual void evalConv(TVector <N> *v, TVector<N> *b) const = 0;
}; 


#endif  // __TSTOKESBASISEQN_HH
