#ifndef __TSTOKESDOFMANAGER_HH
#define __TSTOKESDOFMANAGER_HH
//
//  Project   : libnet
//  File      : TStokesDoFManager.hh
//  Author    : Sabine Gutsch
//  Purpose   : DoFManager for linear conforming elements
//  Date      : 24.07.1998
//  Changes   : 
//  Questions : 
//  ToDo      : 
//
// arch-tag: 74f1b675-39f2-49d4-826f-f3267763eca1
//

#include "TBasisDoFManager.hh"

//
// class for DoF manager for piecewise linear
// elements for the Stokes equations
//
template <int N> 
class TStokesDoFManager : public TBasisDoFManager<N> {
protected:
   uint noOfDoFs[N+1];
   // global counter for DoFs on boundary (for each net)
   uint noOfInnerDoFs[N+1];
   uint noOfBoundaryDoFs[N+1];
public:
	// constructors	
    TStokesDoFManager ();
	// destructor
    ~TStokesDoFManager ();
    // get noOfDofs;  
    uint getNoOfDofs( uint i ) const { return (noOfDoFs[i]); }
	// build stiffness matrix of coarse net, using the fine net  
	TBlockMatrix* buildStiffnessMatrix( TNet<N> *cNet, TNet<N> *fNet = NULL );
private:
    // initialise Dofs
    void initDoFDesc( TNet<N> *cNet, TNet<N> *fNet);
	// get DoF from DoF description of node
    long getDoFName( TVertex<N> *node, uint i );
    // test if there is a dof
    bool hasDoF( TVertex<N> *node, uint i );
	// 2D: test if j'th frontier of 2d-cell has a midnode
	// 3D: test if m'th edge of j'th frontier of 3d-cell has midnode
    bool hasMidNode(TCell<N> *cell, uint j, uint m);
	// 2D: get midnode of j'th frontier of 2d-cell
	// 3D: get midnode of m'th edge of j'th frontier of 3d-cell
    TVertex<N>* getMidNode(TCell<N> *cell, uint j, uint m);
};

#endif  // __TSTOKESDOFMANAGER_HH
