//
//  Project   : libnet
//  File      : stokesHierarchy.hh
//  Author    : Thomas Probst / Sabine Gutsch
//  Purpose   : class to build a matrix and prolongation hierarchy for
//              the Stokes equations
//
// arch-tag: 4c65bc9b-352d-4c33-82b6-2929df0d2494
//

#ifndef __TSTOKESHIERARCHY_HH
#define __TSTOKESHIERARCHY_HH

//
// class to build a matrix and prolongation hierarchy for
// the Stokes equations
//

#include "TBasisHierarchy.hh"
#include "TStokesFE.hh"
#include "TStokesDoFManager.hh"

template<int N>
class TStokesHierarchy : TBasisHierarchy<N> {
protected:
    //pointer for an array for the convective part matrices
    TMatrixBlock **pN; 
    //pointer for an array for the convective part rhs's 
    TBlockVector<TFloat> **pR;  
    uint noOfNets; 
   
public:
    TStokesHierarchy ();
    ~TStokesHierarchy ();

    void buildHierarchy ( TBasisDoFManager<N> *dofManager,
                          TBasisEqn<N> *eqn,
                          TBasisFE<N> *fe,
                          TNetHierarchy<N> *netHier, 
                          TBlockMatrix **pMatrix, 
                          TBlockVector<TFloat> **pRHS, 
                          TBlockVector<TFloat> **pX );

    void updateHierarchy ( TBasisEqn<N> *eqn,
                           TBasisFE<N> *fe,
                           TNetHierarchy<N> *netHier, 
                           TBlockMatrix **pMatrix, 
                           TBlockVector<TFloat> **pRHS, 
                           TBlockVector<TFloat> **pX );
};
    
#endif  // __TSTOKESHIERARCHY
//a do ' make && stokes'
