//
//  Project :  libvisual
//  File    :  TVRML.hh
//  Purpose :  class for a VRML interface
//  Author  :  Ronald Kriemann
//
// arch-tag: a3d6539a-c947-43b1-91f9-7833abe3fadb
//

#ifndef __VRML_HH
#define __VRML_HH

#include "T3DPrinter.hh"

//
// class for a VRML 1.0 interface
//
class TVRML : public T3DPrinter
{
protected:
    // flag to indicate style of drawing
    // (simple and fast / complex and slow)
    static bool bSimpleDraw;
    
public:
	// constructor
	TVRML ( const char *filename );

	// destructor
	virtual ~TVRML ();

    // change between drawing styles
    static void setSimpleDraw ( bool b );
    
	// begin (end) printing
	virtual void begin ();
	virtual void end ();

	// begin (end) new node section 
	// (if name == NULL begin Separator)
	virtual void beginSection ( const char *name = NULL );
	virtual void endSection ();

	//
	// misc. draw commands
	//

    virtual void drawText ( TVector<3> *orig, const char *text );
    
    virtual void drawLine ( TVector<3> *orig, TVector<3> *dest );
    
    virtual void drawCylinder ( TVector<3> *orig, TVector<3> *dest,
                                TFloat radius );

    virtual void drawCone ( TVector<3> *orig, TFloat origRad,
                            TVector<3> *dest, TFloat destRad );

    virtual void drawSphere ( TVector<3> * orig, TFloat radius );
    
    virtual void drawPolygon ();

	// internal transformation
	virtual TFloat transZ ( TVector<3> * v ) const;

    // write color/font def to file
    virtual void writeColor ();
    virtual void writeFont ();

    // calc translation/rotation for given orig/dest
    virtual void calcTrans ( TVector<3> * pOrig, TVector<3> * pDest,
                             TVector<3> * pTrans, TVector<3> * pRotAxis,
                             TFloat * pAngle );
};

#endif  // __VRML_HH
