//
//  Project     : libgraph
//  File        : basegraph.hh
//  Author      : Ronald Kriemann
//  Purpose     : base class for all graphs
//  Date        : 30.06.1997
//
// arch-tag: c61b735a-a037-4cab-bb14-dee80f79c6d0
//

#ifndef __BASEGRAPH_HH
#define __BASEGRAPH_HH

#include "defines.hh"
#include "memory.hh"
#include "list.hh"

#include "basenode.hh"

// different types of a graph (can be or-ed)
#define _DIRECTED_   0x1
#define _PLANAR_     0x2
#define _REDUCIBLE_  0x4
#define _WEIGHTED_   0x8

///////////////////////////////////////////////////
//
// base class for a graph, a list of nodes
//
class TGraph : public TMemory 
{
protected:
	// the nodes in our graph
	TList<TNode>  listNodes;

	// type of a graph
	uint  type;

public:
	// constructor
	TGraph ();

	// destructor
	virtual ~TGraph ();

	// return number of nodes
	virtual const uint getNoOfNodes () const;

	// return the node list
	TList<TNode>& getNodeList ();

	// add a node to the list
	virtual void add ( TNode& node );

	// remove a node from the list
	virtual void remove ( TNode& node );

	// get and set type of graph
	const uint getType () const;
	void setType ( uint t );

	// remove a subgraph (and only subgraphs)
	TGraph& operator-= ( TGraph& subgraph );

	// copy operator
	TGraph& operator= ( TGraph& graph );

	// create new graph
	virtual TGraph* clone ();
	virtual TGraph* create ();

	// output 
	virtual void print ( ostream& os ) const;
	friend ostream& operator<< ( ostream& os, const TGraph& graph );
};

#endif  // __BASEGRAPH_HH
