//
//  Project   : libnet
//  File      : basisHierarchy.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : basis class for creating a hierarchy of 
//              stiffness matrices and prolongations 
//
// arch-tag: edcf8bff-9756-45f8-97d6-e4cdbbca1a3e
//

#ifndef __BASISHIERARCHY_HH
#define __BASISHIERARCHY_HH

#include "memory.hh"

#include "vector.hh"
#include "blockvector.hh"
#include "blockmatrix.hh"

#include "nethierarchy.hh"

#include "basisDoFManager.hh"
#include "basisFE.hh"

//
// basis class for a finite element 
//

class T2DBasisHierarchy : public TMemory {

public:
    // constructor
    T2DBasisHierarchy() {}    
    // destructor
    virtual ~T2DBasisHierarchy() {}    
    // set up hierarchy
    virtual void buildHierarchy ( TBasisDoFManager<2> *dofManager,
                                  T2DBasisEqn *eqn,
                                  T2DBasisFE *fe,
                                  TNetHierarchy<2> *netHier, 
                                  TBlockMatrix **pMatrix, 
                                  TBlockVector<TFloat> **pRHS, 
                                  TBlockVector<TFloat> **pX ) = 0;
};

#endif  // __BASISHIERARCHY_HH
