//
//  Project   : Navier-Stokes
//  File      : cmdline.hh
//  Author    : Ronald Kriemann / Sabine Gutsch
//  Purpose   : evaluation of commandline parameters
//
// arch-tag: f26c5722-1a05-4e70-a2c4-315e7705282c
//

#ifndef __CMDLINE_HH
#define __CMDLINE_HH

#include "defines.hh"
#include "var.hh"
#include "TString.hh"

// multigrid sort methods
#define MGSORT_PROJ    1
#define MGSORT_NESTED  2
#define MGSORT_INDEP   3

class TParameter
{
public:
    uint    startLevel, endLevel;
    uint    noOfNets;
    uint    steps;
    // V or W-cycle or S (only smoothing)
    uint    cycle;
    // number of pre- and post-smoothing steps
    uint    preSmoothing, postSmoothing;
    TString solver, prec;

    // ordering &  criterion for graph
    uint    ordering;
    uint    mgSort;
    uint    criterion; 
    TFloat  xi;
    bool    pblock;
    
    // Red-Black-Ordering
    uint    MaxLevel;
    uint   MaxNoOfBlacks;
    bool    lumping;

    // equation
    TFloat  epsilon;
    bool    reactionTerm;
    TString convdir;
    
    // triangulation
    TString starttri;
    
    // Output
    bool    draw;
    bool    write_matrix;

    // kind of search in graphs (dfs/bfs)
    uint    searchType;

    // for flowsurfaces
    TFloat  fsAlpha, fsBeta, fsGamma, fsKappa;

    // parameter to define strong edges
    TFloat  sigma;

    // flag for building global fvs
    bool    global;

    // max level for cycred
    uint    maxLevel;

    // should we sort by OFD
    bool    sortOFD;
    
    // constructor
    TParameter();

    // read every option and set the parameter
    int evalCmdLine ( int argc, char **argv );

    ////////////////////////////////////////////////////
    //
    // output
    //

    void print();
    friend ostream& operator<< (ostream& os, const TParameter& param);

protected:
    ////////////////////////////////////////////////////
    //
    // read int/float with error handling
    //
    void readInt   ( int & pos, int argc, char **argv, int & n );
    void readInt   ( int & pos, int argc, char **argv, uint & n );
    void readFloat ( int & pos, int argc, char **argv, TFloat & f );
};

// default object for cmdline parsing
extern TParameter param;

#endif  // __CMDLINE_HH
