//
//  Project   : libnet
//  File      : convDirections.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : class for convection directions
//
// arch-tag: e8b7c829-cebc-4c1c-8c0f-704ef55a6495
//

#ifndef __CONVDIRECTIONS_HH
#define __CONVDIRECTIONS_HH

#include "defines.hh"
#include "vector.hh"


//
// no convection
//
TFloat noConv0  (TVector<2> x);
TFloat noConv1  (TVector<2> x);
//
// no reaction
//
TFloat noReac  (TVector<2> x);

//
// xline (x - direction)
//
TFloat xline0  (TVector<2> x);
TFloat xline1  (TVector<2> x);

//
// outboundXline
//
TFloat outboundXline0 (TVector<2> x);
TFloat outboundXline1  (TVector<2> x);

//
// circle centered at (0.5, 0.5)
//
TFloat circle0 ( TVector<2> v );
TFloat circle1 ( TVector<2> v );

// four circles like hotplates
TFloat fourcircles0 ( TVector<2> v );
TFloat fourcircles1 ( TVector<2> v );

//
// curve (quarter of circle centered at (1.0, 1.0))
//
TFloat curve0 ( TVector<2> v );
TFloat curve1 ( TVector<2> v );

//
// curve (quarter of circle centered at (1.0, 1.0))
//
TFloat triangle0 ( TVector<2> v );
TFloat triangle1 ( TVector<2> v );


// square (circle centered at (0.5,0.5), but streched to the corners)
//
TFloat square0 ( TVector<2> v );
TFloat square1 ( TVector<2> v );


// four squares like hotplates
TFloat foursquares0 ( TVector<2> v );
TFloat foursquares1 ( TVector<2> v );

//
// one reaction
//
TFloat oneReac  (TVector<2> x);


// help functions

// v inside the square [a1,a2) x [b1,b2)  ?
bool in(TVector<2> v, TFloat a1,TFloat a2,TFloat b1,TFloat b2);



#endif // __CONVDIRECTIONS_HH


