//
//  Project   : libnet
//  File      : fastGaussSeidel.hh
//  Author    : Sabine Gutsch
//  Purpose   : kaczmarz solver
//
// arch-tag: e5d54e3a-949e-44f0-97de-bf3e0d2235b1
//

#ifndef __FASTGAUSSSEIDEL_HH
#define __FASTGAUSSSEIDEL_HH

#include "solver.hh"

// abbr. for solvers
#define TFFGS  TFastForwardGaussSeidel
#define TFBGS  TFastBackwardGaussSeidel
#define TFSGS  TFastSymmetricGaussSeidel

class TFastForwardGaussSeidel : public TSolver
{
protected:
    TFloat damp;

public:
	// constructor
    TFastForwardGaussSeidel( TBlockMatrix *paA = NULL, 
                             TBlockVector<TFloat> *paRHS = NULL, 
                             TBlockVector<TFloat> *paX = NULL, 
                             uint amaxiter = DEFAULT_MAX_ITERATION,
                             TFloat aaccur = DEFAULT_ACCURACY,
                             uint averb = 0, TFloat aDamp = 1.0 );

	// one iteration step
    int step ();
};

class TFastBackwardGaussSeidel : public TSolver
{
protected:
    TFloat damp;

public:
	// constructor
	TFastBackwardGaussSeidel( TBlockMatrix *paA = NULL, 
                              TBlockVector<TFloat> *paRHS = NULL, 
                              TBlockVector<TFloat> *paX = NULL, 
                              uint amaxiter = DEFAULT_MAX_ITERATION,
                              TFloat aaccur = DEFAULT_ACCURACY,
                              uint averb = 0, TFloat aDamp = 1.0 );

	// one iteration step
    int step ();
};

class TFastSymmetricGaussSeidel : public TSolver
{
protected:
    TFloat damp;

public:
	// constructor
	TFastSymmetricGaussSeidel( TBlockMatrix *paA = NULL, 
                               TBlockVector<TFloat> *paRHS = NULL, 
                               TBlockVector<TFloat> *paX = NULL, 
                               uint amaxiter = DEFAULT_MAX_ITERATION,
                               TFloat aaccur = DEFAULT_ACCURACY,
                               uint averb = 0, TFloat aDamp = 1.0 );

	// one iteration step
    int step ();
};

class TFastBackSymmetricGaussSeidel : public TSolver
{
protected:
    TFloat damp;

public:
	// constructor
	TFastBackSymmetricGaussSeidel( TBlockMatrix *paA = NULL, 
                               TBlockVector<TFloat> *paRHS = NULL, 
                               TBlockVector<TFloat> *paX = NULL, 
                               uint amaxiter = DEFAULT_MAX_ITERATION,
                               TFloat aaccur = DEFAULT_ACCURACY,
                               uint averb = 0, TFloat aDamp = 1.0 );

	// one iteration step
    int step ();
};


//
// class for block lower Gauss-Seidel iteration (based on concentric 
// cycle decomposition)
//
class TFastBlockForwardGS : public TSolver
{
protected:
    TFloat damp;

public:
    // constructor
    TFastBlockForwardGS ( TBlockMatrix *paA = NULL, 
                          TBlockVector<TFloat> *paRHS = NULL, 
                          TBlockVector<TFloat> *paX = NULL,  
                          uint amaxiter = DEFAULT_MAX_ITERATION, 
                          TFloat aaccur = DEFAULT_ACCURACY, 
                          uint averb = 0, TFloat aDamp = 1.0 );
  
    // one iteration step
    int step (); 
}; 

//
// class for block upper Gauss-Seidel iteration (based on concentric 
// cycle decomposition)

//
class TFastBlockBackwardGS : public TSolver
{
protected:
    TFloat damp;

public:
	// constructor
    TFastBlockBackwardGS ( TBlockMatrix *paA = NULL, 
                           TBlockVector<TFloat> *paRHS = NULL, 
                           TBlockVector<TFloat> *paX = NULL, 
                           uint amaxiter = DEFAULT_MAX_ITERATION, 
                           TFloat aaccur = DEFAULT_ACCURACY, 
                           uint averb = 0, TFloat aDamp = 1.0 );
    // one iteration step
    int step ();  
}; 

//////////////////////////////////////////////////
//
// class for sym GS
//

class TFastBlockSymmetricGS : public TSolver
{
protected: 
    TFloat damp;
    TFastBlockForwardGS* For;
    TFastBlockBackwardGS* Back;

public:
    // constructor
    TFastBlockSymmetricGS ( TBlockMatrix *paA = NULL, 
                            TBlockVector<TFloat> *paRHS = NULL, 
                            TBlockVector<TFloat> *paX = NULL, 
                            uint amaxiter = DEFAULT_MAX_ITERATION, 
                            TFloat aaccur = DEFAULT_ACCURACY, 
                            uint averb = 0, TFloat aDamp = 1.0 );
    // destructor
    ~TFastBlockSymmetricGS();

    // one iteration step
    int step (); 
}; 

//
// class for  backward sym GS
//

class TFastBlockBackSymmetricGS : public TSolver
{
protected:
    TFloat damp;
    TFastBlockForwardGS* For;
    TFastBlockBackwardGS* Back;

public:
    // constructor
    TFastBlockBackSymmetricGS ( TBlockMatrix *paA = NULL, 
                                TBlockVector<TFloat> *paRHS = NULL, 
                                TBlockVector<TFloat> *paX = NULL, 
                                uint amaxiter = DEFAULT_MAX_ITERATION, 
                                TFloat aaccur = DEFAULT_ACCURACY, 
                                uint averb = 0, TFloat aDamp = 1.0 );

    ~TFastBlockBackSymmetricGS ();
    
    // one iteration step
    int step (); 
}; 


#endif  // __FASTGAUSSSEIDEL_HH
			
