//
//  Project   : libnet
//  File      : fixedPointDefectCorrection.hh
//  Author    : Sabine Gutsch
//  Purpose   : nonlinear solver class (fixed point defect correction)
//
// arch-tag: c30146c0-e543-46e4-89ec-6ddf858a6a8f
//

#ifndef __FIXEDPOINTDEFECTCORRECTION_HH
#define __FIXEDPOINTDEFECTCORRECTION_HH

#include "nonlinearSolver.hh"

class TFixedPointDefectCorrection : public TNonLinearSolver {

protected:
    TBlockVector<TFloat> vecX;
    TBlockVector<TFloat> vecAu;
    TFloat               steplength;

public:
	// constructor
    TFixedPointDefectCorrection ( TBasisEqn<2> *aEqn, TBasisFE<2> *aFE, 
                           T2DNetHierarchy *aNetHier, 
                           T2DStokesHierarchy *aStokesHier ) 
		: TNonLinearSolver ( aEqn, aFE, aNetHier, aStokesHier )
    {
	    setName("Fixed point defect correction"); 
        steplength = 1.0;
	}

	// destructor
    ~TFixedPointDefectCorrection ( ) {};

    TFloat step ( TMultigrid *mgSolver ) 
    {
        TFloat defectNorm; 
        
        //updateHierarchy( mgSolver );
        vecAu = *(mgSolver->getA()) * *(mgSolver->getX());
        vecX = *(mgSolver->getX());
        mgSolver->getX()->set( 0.0 );
        *(mgSolver->getRHS()) -= vecAu;
 
        defectNorm = mgSolver->getRHS()->weightedEuclidNorm();

        mgSolver->solve();

        *(mgSolver->getX()) *= steplength;
        *(mgSolver->getX()) += vecX;
        *(mgSolver->getRHS()) += vecAu;

        return (defectNorm);
    }

};

#endif  // __FIXEDPOINTDEFECTCORRECTION_HH
