//
//  Project   : libnet
//  File      : fixedPointIteration.hh
//  Author    : Sabine Gutsch
//  Purpose   : nonlinear solver class (fixed point iteration)
//
// arch-tag: caea3bb1-80d0-47fe-9d3e-8e767f098455
//

#ifndef __FIXEDPOINTITERATION_HH
#define __FIXEDPOINTITERATION_HH

#include "nonlinearSolver.hh"

class TFixedPointIteration : public TNonLinearSolver {

public:
	// constructor
    TFixedPointIteration ( TBasisEqn<2> *aEqn, TBasisFE<2> *aFE, 
                           T2DNetHierarchy *aNetHier, 
                           T2DStokesHierarchy *aStokesHier ) 
		: TNonLinearSolver ( aEqn, aFE, aNetHier, aStokesHier )
    {
	    setName("Fixed point iteration"); 
	}

	// destructor
    ~TFixedPointIteration ( ) {};

    TFloat step ( TMultigrid *mgSolver ) 
    {
        TFloat defectNorm; 

        //updateHierarchy( mgSolver );
        mgSolver->solve();
        defectNorm = (*(mgSolver->getA()) * *(mgSolver->getX()) - *(mgSolver->getRHS()) ).weightedEuclidNorm();
        return (defectNorm);
    }

};

#endif  // __FIXEDPOINTITERATION_HH
