#ifndef __FUNCTIONS_HH
#define __FUNCTIONS_HH
//
//  Project   : libnet
//  File      : functions.hh
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : class for compare functions
//
// arch-tag: 17f8aa0d-bd0b-4877-b88f-ecec7a09bac4
//

int
lexComp ( const void * arg0, const void * arg1 )
{
    const TVector<3> * a = (TVector<3>*) arg0;
    const TVector<3> * b = (TVector<3>*) arg1;
    TFloat             eps = 1e-8;
   
    if (fabs((*a)[1] - (*b)[1]) + fabs((*a)[2] - (*b)[2]) < eps) return 0;
    if (((*b)[2] - (*a)[2]) > eps ) return (-1); //  by > ay
    if (((*a)[2] - (*b)[2]) > eps ) return (1);  //  by < ay
    // also: by=ay
    if (((*b)[1] - (*a)[1]) > eps ) return (-1); //  bx > ax
    else return (1);
}

int
cycleComp (const void * arg0, const void * arg1 )
{
    const TVector<3> * a = (TVector<3>*) arg0;
    const TVector<3> * b = (TVector<3>*) arg1;

    TFloat eps = 1e-8;
   	TFloat ra, rb; // 'radius' of circle a, b

    if (fabs((*a)[1] - (*b)[1]) + fabs((*a)[2] - (*b)[2]) < eps) return 0;
    
    ra = MAX( fabs((*a)[1] - 0.5), fabs((*a)[2] - 0.5) );
    rb = MAX( fabs((*b)[1] - 0.5), fabs((*b)[2] - 0.5) );

    if ( rb - ra > eps ) return (-1); //  by > ay
    if ( ra - rb > eps ) return (1);  //  by < ay

    // also: ra = rb
    if ( fabs( (*a)[2] + ra - 0.5) < eps ) {
	// a lies on lower edge
       if ( (fabs( (*b)[2] + ra - 0.5) < eps) && ((*b)[1] < (*a)[1]) ) 
	       return (1);
       else 
           return (-1);
	}
    if ( fabs( (*a)[1] - ra - 0.5 ) < eps ) {
	// a lies on right edge
		if ( fabs( (*b)[1] - ra - 0.5 ) < eps ) {
           if ( (*b)[2] < (*a)[2] ) 
		   // b lies on right edge below a
               return (1);
           else  
               return (-1);
		}
        if ( (fabs( (*b)[2] + ra - 0.5 ) < eps) ) 
		   // b lies on lower edge
           return (1);
        else 
		   // b lies on upper or left edge
           return (-1);
	}
    if ( fabs( (*a)[2] - ra - 0.5) < eps ) {
	// a lies on upper edge
		if ( fabs( (*b)[2] - ra - 0.5) < eps ) {
           if ( (*b)[1] < (*a)[1] ) 
               return (-1);
           else 
               return (1);
		}
        if ( fabs( (*b)[1] + ra - 0.5) < eps )
            return (-1);
        else
            return (1);
	}
    if ( fabs( (*a)[1] + ra - 0.5 ) < eps ) {
	// a lies on left edge
        if ( (fabs( (*b)[1] + ra - 0.5 ) < eps) && ((*b)[2] < (*a)[2]) ) 
            return (-1);
        else
            return (1);
	}
    else {
		cout << "this should never be reached  - only to avoid warning\n";
        return (1); 
	}
}

int nameComp (const TVector<3> *a, const TVector<3> *b) {
    TFloat eps = 1e-8;
    if (fabs((*a)[0] - (*b)[0]) < eps) return (0);  // a.name = b.name 
    if ((*a)[0] < (*b)[0]) return (-1);  //a.name<b.name  
    else
        return (1);    // a.name>b.name
}
  
#endif // __FUNCTIONS_HH
