//
//  Project :  libvisual
//  File    :  graphview.hh
//  Purpose :  class for visualization of a graph
//  Author  :  Ronald Kriemann / Sabine Gutsch
//
// arch-tag: 0276aba6-6cd8-4460-a842-bf057719fb75
//

#ifndef __GRAPHVIEW_HH
#define __GRAPHVIEW_HH

#include "basegraph.hh"

#include "blockvector.hh"

#include "view.hh"

//
// this class gets a graph (where the nodes have geom. info) and draws
// the nodes and edges of this graph
//
class TGraphView : public TView
{
protected:
	// graph we draw
	TGraph  *pGraph;

	// cycles we draw
	TList<TGraph>  *plistCycle;

	// cycles, coverd by cycnodes
	TList<TNode>   *plistCycNodes;

	// distance between the nodes
	TFloat  maxDist, minDist;

	// vectorblock with permuted indices
	TVectorBlock<uint> *pPermuteBlock;

    // draw deleted nodes ?
    bool   bDrawDeleted;
    
public:
	// constructor
	TGraphView ( TGraph& graph );
	TGraphView ( TGraph& graph, TVectorBlock<uint>& permutation );
	TGraphView ( TList<TGraph>& cycles );
	TGraphView ( TList<TGraph>& cycles, TVectorBlock<uint>& permutation );
	TGraphView ( TList<TNode> &listCycNodes );

	// draw the graphview
	void draw ( TPrinter& p );
	void drawGraph ( TPrinter& p );
	void drawCycles ( TPrinter& p );

    // set drawing of deleted nodes
    void setDrawDeleted ( bool b );
    
    // determine minimal distance between nodes in graph
    TFloat minimalDist ( TGraph * pGraph );
};

#endif  // __GRAPHVIEW_HH
