//
//  Project :  libvisual
//  File    :  matrixview.hh
//  Purpose :  class for blockmatrix-visualization
//  Author  :  Ronald Kriemann / Sabine Gutsch
//
// arch-tag: f3f055a3-afe8-4051-9587-4b5e98cfb25e
//

#ifndef __MATRIXVIEW_HH
#define __MATRIXVIEW_HH

#include "matrixblock.hh"
#include "blockmatrix.hh"

#include "view.hh"

// size of an matrixelement
#define ELEMENT_SIZE  8

//
// this class shows the sparsity structure of a matrixblock
//
class TMatrixView : public TView
{
protected:
	// matrix to draw
	TBlockMatrix  * pBlockMatrix;
    TMatrixBlock  * pMatrixBlock;

	// parameter, which chooses the shown values
	TFloat  xi;

	// draw values bigger or smaller than diag
	bool    bDrawSmaller;

    // array holding info about blockstructure
    TDynArray< uint >  * pArrBlockStruct;

    // flag to toggle usage of colors
    static bool _useColors;
    
public:
	// constructor
	TMatrixView ( TBlockMatrix * pMatrix, TFloat x = 0.0, bool smaller = FALSE );
    TMatrixView ( TMatrixBlock * pMatrix, TFloat x = 0.0, bool smaller = FALSE );

    // set usage of colors
    static void useColors ( bool b );
    
    // set blockstructure
    void setBlockStructure ( TDynArray< uint > * pBlockStruct );
    
	// draw matrix
	void draw ( TPrinter & p );

	// return the shown matrixblock
	TMatrixBlock & getMatrixBlock () const;

protected:
    // encapsulate diff. drawing methods
    void drawMatrix ( TPrinter & p, TBlockMatrix * pMatrix );
    void drawBlock  ( TPrinter & p, TMatrixBlock * pMatrix );

    // return maximum entry in matrixblock
    TFloat maxEntry ( TMatrixBlock * pMatrix );

    // check if matrixentry is strong w.r.t. graph-criterion
    bool isStrong ( uint i, uint j, uint bi = 0, uint bj = 0 );
};

#endif  // __MATRIXVIEW_HH
