//
//  Project :  libvisual
//  File    :  dnetview.hh
//  Purpose :  class for net-visualization
//  Author  :  Ronald Kriemann / Sabine Gutsch
//
// arch-tag: 778f31c3-b190-487c-8b7f-375191163e60
//

#ifndef __NETVIEW_HH
#define __NETVIEW_HH

#include "net.hh"
#include "T3DMeshGenerator.hh"
#include "blockvector.hh"

#include "view.hh"

//
// this class goes over the net and draws all cells
//
class TNetView : public TView
{
protected:
	// net, we draw
	T2DNet              * p2DNet;
	T3DNet              * p3DNet;
    T3DMeshGenerator    * pMeshGen;
	uint                 maxInteriorName;

	// vectorblock with permuted indices
	TVectorBlock<uint> * pPermuteBlock;

	// min (max) distance between two nodes in the net
	TFloat                minDistNodes, maxDistNodes;

public:
	// constructor
	TNetView ( T2DNet &net );
	TNetView ( T2DNet &net, TVectorBlock<uint>& permutation );

	TNetView ( T3DNet *net );
	TNetView ( T3DMeshGenerator * mg );

	// draw the net
	virtual void draw ( TPrinter& p );
	virtual void draw3D ( T3DPrinter &p );
    void drawNet ( T3DPrinter & p );
    void drawMG ( T3DPrinter & p );
};

#endif  // __NETVIEW_HH
