//
//  Project     : libgraph
//  File        : nodes.hh
//  Author      : Ronald Kriemann
//  Purpose     : classes for different nodes in a graph
//  Date        : 04.08.1997
//
// arch-tag: 5b48ec63-90e5-4ee4-9255-9bbff1c01075
//

#ifndef __NODES_HH
#define __NODES_HH

#include <limits.h>

#include "vector.hh"

#include "basenode.hh"
#include "basegraph.hh"

// forward decl.
class TPlanarCycle;

////////////////////////////////////////////////
//
// class for a markable node
//

#define NO_MARK  INT_MAX
#define DELETED  INT_MIN

class TMarkableNode : public TNode
{
protected:
	// the mark, this node has
	int  _mark;

public:
	// constructor
	TMarkableNode ( uint n, int m = NO_MARK );
	TMarkableNode ( TMarkableNode& node );

	// access the mark
	virtual const int mark () const;
	virtual void setMark ( int m );

	// create a new nodes
	virtual TMarkableNode* clone ();
	virtual TMarkableNode* create ( uint n );
};

////////////////////////////////////////////////////////////
//
// class for a node which represents a cycle
//
class TCycleNode : public TMarkableNode 
{
protected:
	// pointer to cycle
	TPlanarCycle  *pCycle;

public:
	// constructor
	TCycleNode ( uint n, TPlanarCycle *cycle );

	// destructor
	virtual ~TCycleNode ();

	// access cycle
	virtual TPlanarCycle * cycle ();
	virtual void setCycle ( TPlanarCycle * cycle );

	// create a new nodes
	virtual TCycleNode* clone ();
	virtual TCycleNode* create ( uint n );
};

#endif  // __NODES_HH
