//
//  Project     : libgraph
//  File        : number.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a numbering algorithm
//  Date        : 01.07.1997
//
// arch-tag: ca2294e2-c8d7-4d92-a1df-9f53b9078dfd
//

#ifndef __NUMBER_HH
#define __NUMBER_HH

#include "memory.hh"
#include "blockvector.hh"

#include "graphs.hh"
#include "nodes.hh"

// different marks for a node
#define C        1
#define F        2
#define L        3

// forward decl
class TBlockInfo;

//
// class for the numbering algorithm
//
// If a vector was given, we place all nodes with mark=F at the beginning 
// and all nodes with mark=L at the end. Nodes with mark=C are placed
// in the middle.
//
class TNumberGraph : public TMemory 
{
protected:
	// the graph we number
	TGraph  *pGraph;

	// blockinfo for permutation
	TBlockInfo  *pBlockInfo;

public:
	// constructor
	TNumberGraph ( TGraph& graph, TBlockInfo *blockinfo = NULL );

	// number all nodes
	void number ();

	// set attr F to a node
	void setF ( TMarkableNode& node );

	// set attr L to a node
	void setL ( TMarkableNode& node );
};

#endif  // __NUMBER_HH
