//
//  Project   : libmisc
//  File      : parray.hh
//  Author    : Ronald Kriemann
//  Purpose   : Template class array of pointers
//
// arch-tag: ffef2404-8446-46cc-9c59-9fe897854855
//

#ifndef __PARRAY_HH
#define __PARRAY_HH

#include <string.h>

#include "defines.hh"
#include "memory.hh"

// 
// Template class array (for pointers)
//
// this array dynamically grows if an index is expected greater
// the nSize
//
template <class T> 
class TPArray : public TMemory {

protected:
	T    **pList;       // array to store the data
	uint  nSize;        // number of the element with the max. index
	uint  nRealSize;    // real size of the array
	uint  nGrowBy;      // if index > nSize the list will grow to index+nGrowBy+1

public:
	// public constructors with or withoutout a default size
	// and default size to change
	TPArray () 
	{ 
		nSize = nRealSize = 0;
		nGrowBy = 1;
		pList = 0;
	}
	TPArray ( uint size, uint growby = 0 ) 
	{
		nSize     = 0;
		nGrowBy   = growby; // > 1 ? growby : 1;  // grow at least by one
		nRealSize = size+nGrowBy;

		pList = new T * [nRealSize];
		for (uint i = 0; i < nRealSize; i++)
			pList[i] = NULL;
	}

	// destructor
	~TPArray () {}

	// set the growby size new
	void setGrowBy ( uint growby ) 
	{
		nGrowBy = growby; // > 1 ? growby : 1;  // grow at least by one
	}

	// set a new size for the list and
	// copy all elements from the old list
	void setSize ( uint n ) 
	{
		if (n == nSize) 
			return;
		else if (n == 0) 
		{
			nSize = 0;
			memset( pList, 0, sizeof(T*)*(nRealSize));
		} 
		else if (n > nRealSize)
		{
			nRealSize = n + nGrowBy;
			T **tmpList = new T * [nRealSize];

			if (pList != NULL)
			{
				memcpy( tmpList, pList, sizeof(T*)*nSize );
				delete[] pList;
			}// if
			memset( tmpList+nSize, 0, sizeof(T*)*(nRealSize-nSize));

			nSize = n;
			pList = tmpList;
		} 
		else if (n < nSize - nGrowBy) 
		{
			nSize = n;
			nRealSize = nSize+nGrowBy;
			
			T **tmpList = new T * [nRealSize];

			if (pList != NULL)
			{
				memcpy( tmpList, pList, sizeof(T*)*n );
				delete[] pList;
			}// if
			memset( tmpList+n, 0, sizeof(T*)*(nRealSize-n));
					
			pList = tmpList;
		} 
		else if (n < nSize)
		{
			nSize = n;
			memset( pList+n, 0, sizeof(T*)*(nSize-n));
		} 
	}

	// return the max numer of items stored
	uint getSize () const { return (nSize); }

	// operator [] for index-acessing (extend array if out of bound)
	/*	T* operator [] ( uint i ) {
		if (i >= nRealSize) 
		{
			setSize(i+1);
			nSize = i+1;
		}// if
		return (pList[i]);
	}*/
	T* operator []( uint i ) const 
	{ 
		if (i >= nSize)
		{
//			cerr << "TPArray::operator[] (" << i << ") : out of bound "
//				 << "(max=" << nSize << ")" << endl;
			return (NULL);
		}// if

		return (pList[i]);
	}

	// return object at index i
	T* get ( uint i ) const 
	{
		if (i >= nSize)
		{
//			cerr << "TPArray::get (" << i << ") const : out of bound "
//				 << "(max=" << nSize << ")" << endl;
			return (NULL);
		}// if

		return (pList[i]);
	}
	// set objects at index i
	void set ( uint i, T* object ) 
	{
		if (i >= nRealSize)
		{
			setSize(i+1);
			nSize = i+1;
		}// if
		else if (i >= nSize)
			nSize = i+1;
		pList[i] = object;
	}

	// adds an object to the array
	void add ( T* object ) 
	{
		if (nSize >= nRealSize)
			setSize(nRealSize+1);
		pList[nSize] = object;
		nSize++;
	}

	// clear the list (but not the stored elements)
	void clear () 
	{
		if (pList) delete[] pList;
		pList = NULL;
		nSize = nRealSize = 0;
	}

	// check if given object is in the array
	bool isIn ( T* object ) 
	{
		for (uint i = 0; i < nSize; i++)
			if (pList[i] == object)
				return (TRUE);
		return (FALSE);
	}
};

#endif   // __PARRAY_HH
