//
//  Project :  libvisual
//  File    :  printer.hh
//  Purpose :  class for a ps printer driver
//  Author  :  Ronald Kriemann
//
// arch-tag: 917a43c4-016d-4bd7-828a-69cc75b44cdc
//

#ifndef __PRINTER_HH
#define __PRINTER_HH

#include <stdio.h>

#include "styles.hh"
#include "viewport.hh"

// some paper formats
#define A4  0

//
//
//
class TPrinter
{
protected:
	// file and filename to print
	FILE *pFile;
	char  szFilename[256];

	// transformation object
	TViewport  view;

	// papersize
	TFloat  width, heigth;
	// bordersize
	TFloat  bLeft, bRight, bTop, bBottom;

    // save old settings to prevent unnecessary output
    TFloat  old_r, old_g, old_b;
    TFloat  old_lw;
    char    old_font[64];
    int     old_size;
    
public:
	// constructor
	TPrinter ( const char filename[], uint paper = A4 );

	// destructor
	~TPrinter ();

	// return the viewport
	TViewport& getView ();

	// start (finish) printing
	void begin ();
	void end ();

	// misc drawing commands
	void drawPoint( TFloat x, TFloat y );
	void drawLine( TFloat x1, TFloat y1, TFloat x2, TFloat y2 );
	void drawText( TFloat x, TFloat y, const char text[] );
	void drawArc( TFloat x, TFloat y, TFloat rad,
				  TFloat sang, TFloat eang );
	void drawFilledArc( TFloat x, TFloat y, TFloat rad,
						TFloat sang, TFloat eang );
    void drawArrow ( TFloat x1, TFloat y1,
                     TFloat x2, TFloat y2,
                     TFloat angle = 20, TFloat lengthArrow = (2.0 / 3.0) );

	// misc design commands
	void setLineWidth( TFloat width );
	void setRGBColor( TFloat r, TFloat g, TFloat b );
	void setRGBColor( uint rgb );
	void setHSVColor( TFloat h, TFloat s, TFloat v );
	void setFont( const char font[], int size );
};

#endif  // __PRINTER_HH
