//
//  Project   : libnet
//  File      : residualSmoother.hh
//  Author    : Sabine Gutsch 
//  Purpose   : smoother for an indefinite system 
//              following Fischer, Polynomial based iteration
//              methods for symmetric linear systems, p.189
//
// arch-tag: e6e9f837-0930-4812-b822-39d94f87a0fd
//

#ifndef __RESIDUALSMOOTHER_HH
#define __RESIDUALSMOOTHER_HH

#include "solver.hh"

class TResidualSmoother : public TSolver {
 
protected:
	// basic iteration
    TSolver  *pBasicIteration;
    TBlockVector<TFloat> r, rt, xt, h;
    TFloat   eta;     
public:
	// constructor
    TResidualSmoother ( TSolver *pBasicIter, TBlockMatrix *paA = NULL, 
             TBlockVector<TFloat> *paRHS = NULL, 
             TBlockVector<TFloat> *paX = NULL, 
             uint amaxiter = DEFAULT_MAX_ITERATION,
             TFloat aaccur = DEFAULT_ACCURACY, 
             uint averb = 0 );

	// initialize residual
    void init();
    // one iteration step
    int step();
	// set a new basic iteration
	void setBasicIter ( TSolver *pBasicIter );
}; 

#endif  // __RESIDUALSMOOTHER_HH
