//
//  Project   : libmisc
//  File      : scanner.hh
//  Author    : Ronald Kriemann
//  Purpose   : class for a lex. scanner
//
// arch-tag: 19351e51-e833-4aa1-9032-0b74da359c2b
//

#ifndef __SCANNER_HH
#define __SCANNER_HH

#include "memory.hh"

// possible tokens
enum TToken { tokNothing, tokEOL, 
              tokInt, tokFloat, 
              tokVertex, tokFrontier, tokCell,
              tokInterior, tokDirichlet, tokNeumann,
              tokBaensch, tokRegular
};

//
// this scanner take a string and scans for given types 
// (vertex, frontier etc). if one is found, the token and value 
// is returned in the given object data
//
class TScanner : TMemory
{
public:
    // class for dataexchange
    class TTokenData
    {
    public:
        // wich token
        TToken  token;

        // data of this token
        union
        {
            int    nInt;
            TFloat fFloat;
            char   szName[32];
        };
    };

protected:
    // string which we scan
    char  szString[256];

    // act. position in string
    int   nPos;

public:
    // constructor
    TScanner ( const char *string );

    // returns next token and sets value of token
    int nextToken ( TTokenData& data );

    // tries to read an integer and gives it back
    int getInt ( int &nInt );

    // converts alpha char in digit
    int alpha2digit ( char alpha );
};

#endif  // __SCANNER_HH
