//
//  Project     : libnet
//  File        : sparsevector.hh
//  Author      : Ronald Kriemann
//  Purpose     : class for a sparse blockvector
//
// arch-tag: 35edbe7c-8302-4002-a14b-7bcb02402394
//

#ifndef __SPARSEVECTOR_HH
#define __SPARSEVECTOR_HH

#include <limits.h>

#include "memory.hh"
#include "defines.hh"

#include "blockvector.hh"
#include "TMatrixData.hh"

////////////////////////////////////////////////////////////
//
// TSparseBlockVector : class for a sparse blockvector
//
class TSparseBlockVector : public TDynArray<TMatrixData>
{
public:
    // constructor
    // init an empty vector
    TSparseBlockVector ();
    // init an empty vector with n blocks
    TSparseBlockVector ( int n );
    // init an vector with given number of indices in n blocks
    TSparseBlockVector ( int n, uint size[] );
    // copy constructor
    TSparseBlockVector ( const TSparseBlockVector& vec );

    // destructor
    ~TSparseBlockVector ();

    // set a new number of blocks
    void setNoOfBlocks ( uint n );

    // add operator (we assume, that both vectors have same structure)
    TSparseBlockVector& operator+= ( const TSparseBlockVector& vec );
    TSparseBlockVector operator+ ( const TSparseBlockVector& vec );

    // sub operator (we assume, that both vectors have same structure)
    TSparseBlockVector& operator-= ( const TSparseBlockVector& vec );
    TSparseBlockVector operator- ( const TSparseBlockVector& vec );

    // scalar mult
    TSparseBlockVector& operator*= ( const TFloat& scalar );
    TSparseBlockVector operator* ( const TFloat& scalar );
    friend TSparseBlockVector operator* ( const TFloat& scalar,
                                          const TSparseBlockVector& vec );

    // inner produkt between sparsevectors (we assume, that both
    // vectors have same structure)
    TFloat operator* ( const TSparseBlockVector& vec );

    // copy operator
    TSparseBlockVector& operator= ( const TSparseBlockVector& vec );

    // ouput
    friend ostream& operator<< (ostream& os, const TSparseBlockVector& vec);
};

#endif  // __SPARSEVECTOR_HH
