//
//  Project   : libnet
//  File      : stokesFE.hh
//  Author    : Thomas Probst / Sabine Gutsch
//  Purpose   : class for linear conforming 2D finite elements
//
// arch-tag: 8c41ef98-99a4-46dd-8c68-f611b5b1d41d
//

#ifndef __STOKESFE_HH
#define __STOKESFE_HH

#include "basisFE.hh"

//
// class for a conforming linear finite element 
//

class T2DStokesFE : public T2DBasisFE {

protected:
	// number of children
    uint  noOfChildren;
	// gradients of basis functions on children cells
    TVector<2> gradT[4][3];
    // determinants of transformations for children cells
    TFloat     detJ[4];
    // sum of values of coarse basis functions on children cells
    TFloat     c[4][3];
	// dofs for u, p
    uint       dof_u[4][3], dof_p[3];
	// coordinates of vertices of children cells
    TVector<2> coord[4][3], pCoord[3];   
    // boundary names are larger than BOUNDARY_NAME
	uint BOUNDARY_NAME; 
    // number of interiour nodes in coarse grid 
    uint cInterior;     

public:
	// constructors	
    T2DStokesFE ( TDiscrType aDiscrType = Null);
	
	// destructor
    ~T2DStokesFE ();

	// get coordinates of vertices of children cells
    void getCoord ( TCell<2> *cell );

	// get coordinates of all vertices and write them into global vector
   //  void getCoord ( TNet<2> *net, T2DVector **coord );
	void getCoord ( TNet<2> *net, TDynArray< TVector<2> * >& arrCoordU );
	void getCoord ( TNet<2> *net, TDynArray< TVector<2> * >& arrCoordU,
                    TDynArray< TVector<2> * >& arrCoordP  );
   
	void getVertices ( TNet<2> *net, T2DVertex **VerU );
   
	// initialize grad, det, c, dof_u and dof_p for a particular cell
   void initDiscr ( TCell<2> *cell );
   
	// calculate global stiffnes matrix
    void discrDiff ( T2DBasisEqn *eqn, TNet<2> *net, TBlockMatrix *A, 
                    TBlockVector<TFloat> *rhs );
   
	// calculate stiffness matrix of non-symmentric part
   void discrConv ( T2DBasisEqn *eqn, TNet<2> *net, TMatrixBlock *N, 
                    TBlockVector<TFloat> *rhs );
   
    // calculate element stiffness matrix
    void discrDiff ( T2DBasisEqn *eqn, TBlockMatrix *A, 
                    TBlockVector<TFloat> *rhs );
    void discrUP ( T2DBasisEqn *eqn, TBlockMatrix *A,  
                  TBlockVector<TFloat> *rhs );
    void discrConv ( T2DBasisEqn *eqn, TMatrixBlock *N, 
                    TBlockVector<TFloat> *rhs );
    void discrConv ( T2DBasisEqn *eqn, TMatrixBlock *N, 
                    TBlockVector<TFloat> *rhs,  
                    TVectorBlock<TFloat> &suppSize );
	void discrMass(T2DBasisEqn *, TNet<2> *, 
                           TVectorBlock<TFloat> *);
    void  discrMass ( T2DBasisEqn *eqn, TVectorBlock<TFloat> *M);
    void getSuppSize ( TVectorBlock<TFloat> &suppSize );
   
};

#endif  // __STOKESFE_HH
