//
//  Project   : libnet
//  File      : stokesHierarchy.hh
//  Author    : Thomas Probst / Sabine Gutsch
//  Purpose   : class to build a matrix and prolongation hierarchy for
//              the Stokes equations
//
// arch-tag: a4fb780d-a819-427f-b7eb-a00cd1afaf61
//

#ifndef __STOKESHIERARCHY_HH
#define __STOKESHIERARCHY_HH

//
// class to build a matrix and prolongation hierarchy for
// the Stokes equations
//

#include "basisHierarchy.hh"
#include "stokesFE.hh"
#include "stokesDoFManager.hh"

class T2DStokesHierarchy : T2DBasisHierarchy {
protected:
    //pointer for an array for the convective part matrices
    TMatrixBlock **pN; 
    //pointer for an array for the convective part rhs's 
    TBlockVector<TFloat> **pR;  
    uint noOfNets; 
   
public:
   
    T2DStokesHierarchy ();

    ~T2DStokesHierarchy ();

    void buildHierarchy ( TBasisDoFManager<2> *dofManager,
                          T2DBasisEqn *eqn,
                          T2DBasisFE *fe,
                          TNetHierarchy<2> *netHier, 
                          TBlockMatrix **pMatrix, 
                          TBlockVector<TFloat> **pRHS, 
                          TBlockVector<TFloat> **pX );

    void updateHierarchy ( T2DBasisEqn *eqn,
                           T2DBasisFE *fe,
                           TNetHierarchy<2> *netHier, 
                           TBlockMatrix **pMatrix, 
                           TBlockVector<TFloat> **pRHS, 
                           TBlockVector<TFloat> **pX );
};
    
#endif  // __STOKESHIERARCHY
//a do ' make && stokes'
