//
//  Project :  libvisual
//  File    :  streakline.hh
//  Purpose :  class for streakline visualization
//  Author  :  Ronald Kriemann
//
// arch-tag: 246a003f-e8f9-47a9-a7ce-3f7181e82407
//

#ifndef __STREAKLINE_HH
#define __STREAKLINE_HH

#include "memory.hh"
#include "list.hh"

#include "basisEqn.hh"

#include "net.hh"

#include "vector.hh"
#include "blockvector.hh"

#include "view.hh"

//
// class for a particle
//
class TParticle : public TMemory
{
protected:
	// position of the particle
	T2DVector   vecPos;

public:
	// constructor
	TParticle ( const T2DVector &pos );

	// return pos.
	const T2DVector& getPos () const;

	// compute next pos. of particle
	TParticle* calcNextStep ( TFloat deltaT, T2DNet &net,
							  TBlockVector<TFloat> &arrVelocity,
							  T2DBasisEqn &eqn );
};

//
//
// class for a particletrace
//
class TParticleTrace : public TMemory
{
protected:
	// the discrete domain
	T2DNet                *pNet;

	// vector with velocities
	TBlockVector<TFloat>  *parrVelocity;

	// list for trace of particle
	TList<TParticle>       listParticles;

	// eqn. for boundaryvalues
	T2DBasisEqn           *pEqn;

public:
	// constructor
	TParticleTrace ( TParticle &particle,
					 T2DNet &net, TBlockVector<TFloat> &velocity,
					 T2DBasisEqn &eqn );

	// destructor
	~TParticleTrace ();

	// return list
	const TList<TParticle>& getList () const;

	// compute next pos. of all particles
	void calcNextStep ( TFloat deltaT );
};

//
// class for streakline visualization
//
class TStreakLineView : public TView
{
protected:
	// list of particletraces
	TList<TParticleTrace>  listTraces;

public:
	// constructor
	TStreakLineView ( T2DNet &net, TBlockVector<TFloat> &velocity,
					  T2DBasisEqn &eqn );

	// destructor
	virtual ~TStreakLineView ();

	// build traces
	void buildTraces ( TFloat deltaT, uint intervals );

	// draw traces on screen
	virtual void draw ( TPrinter& p );
};

#endif  // __STREAKLINE_HH
