//
//  Project    : libnet
//  File       : vertex.hh
//  Author     : Sabine Gutsch / Ronald Kriemann 
//  Purpose    : classes for 2d and 3d vertices
//
// arch-tag: ce3972cd-28f6-4770-83c3-05b08d279bff
//

#ifndef __VERTEX_HH
#define __VERTEX_HH

extern "C" {
#include <limits.h>
}

#include "vector.hh"

#include "TBoundary.hh"
#include "TDoFDesc.hh"

#define BOUNDARY_VERTEX   ULONG_MAX

// desc for interior nodes
#define INTERIOR  0

//
// class for a template (in dim) vertex 
//
template <int N>
class TVertex : public TVector<N>, public TMultiReference 
{
protected:
    uint         name;

    // pointer to boundary (NULL for interior nodes)
    TBoundary    *pBoundary;

    // desc of DoF's (assoc. names) for boundary nodes
    TDoFDesc     *pDoFDesc;
    
public:
    // constructors and destructor
    TVertex ( TFloat x, TFloat y, uint vtype = 0 );
    TVertex ( TFloat x, TFloat y, TFloat z, TBoundary *boundary );
    TVertex ( const TVector<2> &vector, uint vtype = 0 );
    TVertex ( const TVector<3> &vector, TBoundary *boundary );

    ~TVertex ();
    
    // get names
    const uint getName () const;
    static uint getMaxInteriorName ();
    static uint getMinBoundaryName ();
    static void resetNames ();

    // access boundary
    TBoundary * getBoundary () const;

    // access DoFDesc
    TDoFDesc  * getDoFDesc () const;
    void initDoFDesc ( int m );
};

///////////////////////////////////////////////////////////
//
// implementation
//

//
// some typedefs
//
typedef  TVertex<2>  T2DVertex;
typedef  TVertex<3>  T3DVertex;

#endif  // __VERTEX_HH
