//
//  Project   : libnet
//  File      : TMatrix.cc
//  Author    : tpr
//  Purpose   : template class for a vector (template in dimension)
//
// arch-tag: 136c6734-98d6-479f-af70-fd8473cc61e9
//

#include "TMatrix.hh"

//
// constructors
//
template <int N>
TMatrix<N>::TMatrix ()
{
//    for (uint i=0;i<N;i++)
//       rows[i] = 0;
}


//
// add-and-set operators
//


template <int N>
TMatrix<N>&  
TMatrix<N>::operator+= ( const TMatrix<N>& matrix  ) {
   for (uint i=0;i<N;i++)
      rows[i]+= matrix.rows[i];
   return (*this);
}

//
// sub-and-set operators
//


template <int N>
TMatrix<N>& 
TMatrix<N>::operator-= ( const TMatrix<N>& matrix ) {
   for (uint i=0;i<N;i++)
      rows[i] -= matrix.rows[i];
   return (*this);
}

//
// scalar multiplication
//

template <int N>
TMatrix<N>& 
TMatrix<N>::operator*= ( TFloat scalar ) 
{ 
   for (uint i=0;i<N;i++)
      rows[i] *= scalar;
   return (*this);
}

//
// matrix-vector multiplication
//
TVector<2> 
TMatrix<2>::operator * ( const TVector<2> & vec ) 
{
    TVector<2>  vecTmp;
    vecTmp[0] = (rows[0]*vec);
    vecTmp[1] = (rows[1]*vec);
    return vecTmp;
}

TVector<3> 
TMatrix<3>::operator * ( const TVector<3> & vec ) 
{
    TVector<3>  vecTmp;
    vecTmp[0] = (rows[0]*vec);
    vecTmp[1] = (rows[1]*vec);
    vecTmp[2] = (rows[2]*vec);
    return vecTmp;
}

TVector<4> 
TMatrix<4>::operator * ( const TVector<4> & vec )
{
    TVector<4>  vecTmp;
    vecTmp[0] = (rows[0]*vec);
    vecTmp[1] = (rows[1]*vec);
    vecTmp[2] = (rows[2]*vec);
    vecTmp[3] = (rows[3]*vec);
    return vecTmp;
}   

//
//
// array operator
//

template <int N>
TVector<N>& 
TMatrix<N>::operator[] ( uint i ) 
{ 
#ifdef ERR_BOUNDS
   if (i >= N)
      cerr << "TMatrix::operator[] (" << i << ") : "
           << "out of bound (dim=" << N << ")" << endl;
#endif
   return (rows[i]); 
}

template <int N>
const TVector<N>& 
TMatrix<N>::operator[] ( uint i ) const 
{ 
#ifdef ERR_BOUNDS
   if (i >= N)
      cerr << "const TMatrix::operator[] (" << i << ") const : "
           << "out of bound (dim=" << N << ")" << endl;
#endif
   return (rows[i]); 
}

//
// copy operator
//
template <int N>
TMatrix<N>&
TMatrix<N>::operator= ( const TMatrix<N>& matrix )
{
   for (uint i=0;i<N;i++)
      rows[i] = matrix.rows[i];
   return (*this);
}

//
// comparing
//
template <int N>
const bool 
TMatrix<N>::operator == ( const TMatrix<N> &matrix ) const
{ bool flag=FALSE;

for (uint i=0;i<N;i++)
   if (rows[i] == matrix.rows[i])
      flag=TRUE;
  
return flag;
}

template <int N>
const bool 
TMatrix<N>::operator != ( const TMatrix<N> &matrix ) const
{
   return !(*this == matrix);
}

//
// ouput
//
template <int N>
ostream& 
operator << ( ostream & os, const TMatrix<N> & matrix ) 
{
   os.setf( ios::fixed, ios::floatfield);
   os.precision(2);
   for (uint i=0;i<N;i++) 
      os << " ( " << matrix[i]  << " ) "<< endl;
   return (os);
   
}

ostream & 
operator <<  (ostream & os, const TMatrix<3> & matrix) 
{
   os.setf( ios::fixed, ios::floatfield);
   os.precision(2);
   for (uint i=0;i<3;i++) 
      os << " ( " << matrix[i]  << " ) "<< endl;
   return (os);
   
}
ostream & 
operator <<  (ostream & os, const TMatrix<2> & matrix) 
{
   os.setf( ios::fixed, ios::floatfield);
   os.precision(2);
   for (uint i=0;i<2;i++) 
      os << " ( " << matrix[i]  << " ) "<< endl;
   return (os);
   
}
ostream & 
operator <<  (ostream & os, const TMatrix<4> & matrix) 
{
   os.setf( ios::fixed, ios::floatfield);
   os.precision(2);
   for (uint i=0;i<4;i++) 
      os << " ( " << matrix[i]  << " ) "<< endl;
   return (os);
   
}


TMatrix<2> 
TMatrix<2>::inverted()
{
    TMatrix<2>  matTmp;
    TFloat      a,b,c,d,det;

    // M= [ a b // c d]
    a = (*this)[0][0];
    b = (*this)[0][1];
    c = (*this)[1][0];
    d = (*this)[1][1];
    
    det=a*d-c*b;
    
    matTmp[0][0] =  d/det;
    matTmp[1][0] = -c/det;
    matTmp[0][1] = -b/det;
    matTmp[1][1] =  a/det;

    return matTmp;
} 

   


TMatrix<3> 
TMatrix<3>::inverted()
{
    TMatrix<3>  matTmp;
    TFloat      a,b,c,d,e,f,g,h,i,det;

    // M= [ a b c // d e f // g h i ]
    a=(*this)[0][0];b=(*this)[0][1]; c=(*this)[0][2];
    d=(*this)[1][0];e=(*this)[1][1]; f=(*this)[1][2];
    g=(*this)[2][0];h=(*this)[2][1]; i=(*this)[2][2];
    
    det = a*(e*i-f*h) -b*(d*i-g*f)+c*(d*h-g*e);
    
    matTmp[0][0] =  (e*i-h*f)/det;
    matTmp[1][0] = -(d*i-g*f)/det;
    matTmp[2][0] =  (d*h-g*e)/det;
    
    matTmp[0][1] = -(b*i-h*c)/det;
    matTmp[1][1] =  (a*i-g*c)/det;
    matTmp[2][1] = -(a*h-g*b)/det;
    
    matTmp[0][2] =  (b*f-e*c)/det;
    matTmp[1][2] = -(a*f-d*c)/det;
    matTmp[2][2] =  (a*e-d*b)/det;

    return matTmp;
} 


template <int N>
TMatrix<N> 
TMatrix<N>::inverted()
{
    TMatrix<N>  matTmp( *this );
    
    cerr << "TMatrix<N>::inverted () : sorry, not implemented for N = " << N << endl;
    
    return matTmp;
}   

// to initialize the used templates
template class TMatrix<2>;
template class TMatrix<3>;
template class TMatrix<4>;

