//
//  Project   : libmisc
//  File      : memory.cc
//  Author    : Ronald Kriemann
//  Purpose   : basic class for object counting
//
// arch-tag: 93382138-2eb3-4d1a-88a9-01fb1f5e9095
//


extern "C" {
#include <stdio.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <unistd.h>
}

#include "defines.hh"
#include "memory.hh"

// definition
uint TMemory::nNoOfAlloc   = 0;
uint TMemory::nNoOfDealloc = 0;

//
// print statistics
//
void 
TMemory::statistics()
{
	cout << "Number of alloc. Objects : " << nNoOfAlloc << endl
		 << "Number of deall. Objects : " << nNoOfDealloc << endl
		 << "Number of active Objects : " << nNoOfAlloc-nNoOfDealloc << endl;
}

void 
TMemory::statActive ()
{
	cout << "Number of active Objects : " << nNoOfAlloc-nNoOfDealloc << endl;
}

//
// return used memory (in kbytes)
//
uint
TMemory::used ()
{
    uint   memSize = 0;
    
#ifdef LINUX
    
    //
    // read size as first number in "/proc/<pid>/statm"
    //
    static char  buf[256];
    FILE        *file;
    uint         pages;

    sprintf( buf, "/proc/%d/statm", uint(getpid()) );

    if ((file = fopen( buf, "r" )) == NULL)
    {
        perror( "TMemory::used () : open" );
        return 0;
    }// if

    fgets( buf, sizeof(buf), file );

    fclose( file );

    sscanf( buf, "%d", & pages );
    memSize = pages * getpagesize();
    
#elif SUNOS
    
    //
    // read size of file "/proc/<pid>/as"
    //
    struct stat  stat_buf;
    static char  buf[256];

    sprintf( buf, "/proc/%d/as", uint(getpid()) );
    
    if (stat( buf, & stat_buf ) < 0)
    {
        perror( "TMemory::used () : stat" );
        return 0;
    }// if
    
    memSize = stat_buf.st_size;
    
#else
    
    cout << "TMemory::used () : meminfo unsupported" << endl;
    return 0;
    
#endif

    return memSize / 1024;
}
