//
//  Project   : libnet
//  File      : T3DMeshGenerator.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for an a simple meshgenerator in 3D
//
// arch-tag: 9fb4cbfe-31e7-4d8f-9c94-8008b7d9c2db
//

#include "drawapp.hh"

#include "T3DTriangle.hh"
#include "T3DTetrahedron.hh"

#include "T3DMeshGenerator.hh"

// defines for neighbour access
#define LEFT   0
#define RIGHT  1
#define FRONT  2
#define BACK   3
#define UP     4
#define DOWN   5

///////////////////////////////////////////
///////////////////////////////////////////
//
// class for representing a quad
//
///////////////////////////////////////////
///////////////////////////////////////////

///////////////////////////////////////////
//
// constructors and destructor
//

T3DQuad::T3DQuad ( TDomain * domain )
{
    initData();
    
    pDomain = domain;

    if (pDomain == NULL)
    {
        cout << "T3DQuad::T3DQuad (...) : have NULL domain" << endl;
        exit( 1 );
    }// if
}

T3DQuad::T3DQuad ( TDomain * domain,
                   TVertex<3> * v0, TVertex<3> * v1,
                   TVertex<3> * v2, TVertex<3> * v3,
                   TVertex<3> * v4, TVertex<3> * v5,
                   TVertex<3> * v6, TVertex<3> * v7 )
{
    initData();
    
    pDomain = domain;

    if (pDomain == NULL)
    {
        cout << "T3DQuad::T3DQuad (...) : have NULL domain" << endl;
        exit( 1 );
    }// if

    arrVertices[0] = v0;
    arrVertices[1] = v1;
    arrVertices[2] = v2;
    arrVertices[3] = v3;
    arrVertices[4] = v4;
    arrVertices[5] = v5;
    arrVertices[6] = v6;
    arrVertices[7] = v7;
}
               
T3DQuad::~T3DQuad ()
{
}

///////////////////////////////////////////
//
// access member variables
//

void
T3DQuad::initData ()
{
    uint  i;
    
    arrNeighbours[LEFT]  = arrNeighbours[RIGHT] = NULL;
    arrNeighbours[FRONT] = arrNeighbours[BACK]  = NULL;
    arrNeighbours[UP]    = arrNeighbours[DOWN]  = NULL;

    arrVertices.setSize( 8 );
    arrEdges.setSize( 19 );
    arrFrontiers.setSize( 18 );
    arrCells.setSize( 6 );

    for ( i = 0; i < 8; i++ )
        arrVertices[i] = NULL;

    for ( i = 0; i < 19; i++ )
        arrEdges[i] = NULL;

    for ( i = 0; i < 18; i++ )
        arrFrontiers[i] = NULL;

    for ( i = 0; i < 6; i++ )
        arrCells[i] = NULL;
}
    
T3DQuad *
T3DQuad::neighbour ( uint i )
{
    if (i < 6)
        return arrNeighbours[i];
    else
    {
        cout << "T3DQuad::neighbour (...) : incorrect index" << endl;
        return NULL;
    }// else
}

void
T3DQuad::setNeighbour( uint i, T3DQuad * pNeighbour )
{
    if (i < 6)
        arrNeighbours[i] = pNeighbour;
    else
        cout << "T3DQuad::setNeighbour (...) : incorrect index" << endl;
}

void
T3DQuad::setNeighbours ( TList< T3DQuad > * quads )
{
    TListIterator< T3DQuad >   iter;
    T3DQuad                  * pQuad;

    iter.setList( * quads );
    iter.first();
    while ((pQuad = iter++) != NULL)
    {
        if (pQuad == this)
            continue;

        if (arrNeighbours[LEFT] == NULL)
        {
            if ((arrVertices[0] == pQuad->vertex( 1 )) &&
                (arrVertices[3] == pQuad->vertex( 2 )) &&
                (arrVertices[4] == pQuad->vertex( 5 )) &&
                (arrVertices[7] == pQuad->vertex( 6 )))
            {
                arrNeighbours[LEFT] = pQuad;
//                pQuad->setNeighbour( RIGHT, this );

                continue;
            }// if
        }// if
        
        if (arrNeighbours[RIGHT] == NULL)
        {
            if ((arrVertices[1] == pQuad->vertex( 0 )) &&
                (arrVertices[2] == pQuad->vertex( 3 )) &&
                (arrVertices[5] == pQuad->vertex( 4 )) &&
                (arrVertices[6] == pQuad->vertex( 7 )))
            {
                arrNeighbours[RIGHT] = pQuad;
//                pQuad->setNeighbour( LEFT, this );

                continue;
            }// if
        }// if
        
        if (arrNeighbours[FRONT] == NULL)
        {
            if ((arrVertices[0] == pQuad->vertex( 3 )) &&
                (arrVertices[1] == pQuad->vertex( 2 )) &&
                (arrVertices[4] == pQuad->vertex( 7 )) &&
                (arrVertices[5] == pQuad->vertex( 6 )))
            {
                arrNeighbours[FRONT] = pQuad;
//                pQuad->setNeighbour( BACK, this );

                continue;
            }// if
        }// if
        
        if (arrNeighbours[BACK] == NULL)
        {
            if ((arrVertices[3] == pQuad->vertex( 0 )) &&
                (arrVertices[2] == pQuad->vertex( 1 )) &&
                (arrVertices[7] == pQuad->vertex( 4 )) &&
                (arrVertices[6] == pQuad->vertex( 5 )))
            {
                arrNeighbours[BACK] = pQuad;
//                pQuad->setNeighbour( FRONT, this );

                continue;
            }// if
        }// if
        
        if (arrNeighbours[UP] == NULL)
        {
            if ((arrVertices[4] == pQuad->vertex( 0 )) &&
                (arrVertices[5] == pQuad->vertex( 1 )) &&
                (arrVertices[6] == pQuad->vertex( 2 )) &&
                (arrVertices[7] == pQuad->vertex( 3 )))
            {
                arrNeighbours[UP] = pQuad;
//                pQuad->setNeighbour( DOWN, this );

                continue;
            }// if
        }// if
        
        if (arrNeighbours[DOWN] == NULL)
        {
            if ((arrVertices[0] == pQuad->vertex( 4 )) &&
                (arrVertices[1] == pQuad->vertex( 5 )) &&
                (arrVertices[2] == pQuad->vertex( 6 )) &&
                (arrVertices[3] == pQuad->vertex( 7 )))
            {
                arrNeighbours[DOWN] = pQuad;
//                pQuad->setNeighbour( UP, this );

                continue;
            }// if
        }// if
    }// while
}

void
T3DQuad::checkNeighbours ()
{
    T3DQuad  * pQuad;
    
    if (arrNeighbours[LEFT] != NULL)
    {
        pQuad = arrNeighbours[LEFT];
        
        if ( ! ((arrVertices[0] == pQuad->vertex( 1 )) &&
                (arrVertices[3] == pQuad->vertex( 2 )) &&
                (arrVertices[4] == pQuad->vertex( 5 )) &&
                (arrVertices[7] == pQuad->vertex( 6 ))) )
            cout << "T3DQuad::checkNeighbours : wrong left neighbour" << endl;

        if ( ! (( arrEdges[10] == pQuad->edge( 1 )) &&
                ( arrEdges[12] == pQuad->edge( 3 )) &&
                ( arrEdges[14] == pQuad->edge( 4 )) &&
                ( arrEdges[17] == pQuad->edge( 7 )) &&
                ( arrEdges[18] == pQuad->edge( 6 )) ) )
            cout << "T3DQuad::checkNeighbours : wrong left edges" << endl;
    }// if
        
    if (arrNeighbours[RIGHT] != NULL)
    {
        pQuad = arrNeighbours[RIGHT];
        
        if ( ! ((arrVertices[1] == pQuad->vertex( 0 )) &&
                (arrVertices[2] == pQuad->vertex( 3 )) &&
                (arrVertices[5] == pQuad->vertex( 4 )) &&
                (arrVertices[6] == pQuad->vertex( 7 ))) )
            cout << "T3DQuad::checkNeighbours : wrong right neighbour" << endl;

        if ( ! ((arrEdges[1] == pQuad->edge( 10 )) &&
                (arrEdges[3] == pQuad->edge( 12 ))  &&
                (arrEdges[4] == pQuad->edge( 14 ))  &&
                (arrEdges[6] == pQuad->edge( 18 ))  &&
                (arrEdges[7] == pQuad->edge( 17 )) ) )
            cout << "T3DQuad::checkNeighbours : wrong right edges" << endl;
    }// if
        
    if (arrNeighbours[FRONT] != NULL)
    {
        pQuad = arrNeighbours[FRONT];
        
        if ( ! ((arrVertices[0] == pQuad->vertex( 3 )) &&
                (arrVertices[1] == pQuad->vertex( 2 )) &&
                (arrVertices[4] == pQuad->vertex( 7 )) &&
                (arrVertices[5] == pQuad->vertex( 6 ))) )
            cout << "T3DQuad::checkNeighbours : wrong front neighbour" << endl;

        if ( ! ((arrEdges[0] == pQuad->edge( 16 ))  &&
                (arrEdges[1] == pQuad->edge( 6 ))  &&
                (arrEdges[2] == pQuad->edge( 15 ))  &&
                (arrEdges[9] == pQuad->edge( 13 ))  &&
                (arrEdges[10] == pQuad->edge( 18 )) ) )
            cout << "T3DQuad::checkNeighbours : wrong front edges" << endl;
    }// if
        
    if (arrNeighbours[BACK] != NULL)
    {
        pQuad = arrNeighbours[BACK];
        
        if ( ! ((arrVertices[3] == pQuad->vertex( 0 )) &&
                (arrVertices[2] == pQuad->vertex( 1 )) &&
                (arrVertices[7] == pQuad->vertex( 4 )) &&
                (arrVertices[6] == pQuad->vertex( 5 ))) )
            cout << "T3DQuad::checkNeighbours : wrong back neighbour" << endl;
        
        if ( ! ((arrEdges[6] == pQuad->edge( 1 ))  &&
                (arrEdges[13] == pQuad->edge( 9 ))  &&
                (arrEdges[15] == pQuad->edge( 2 ))  &&
                (arrEdges[16] == pQuad->edge( 0 ))  &&
                (arrEdges[18] == pQuad->edge( 10 ))) )
            cout << "T3DQuad::checkNeighbours : wrong back edges" << endl;
    }// if
        
    if (arrNeighbours[UP] != NULL)
    {
        pQuad = arrNeighbours[UP];
        
        if ( ! ((arrVertices[4] == pQuad->vertex( 0 )) &&
                (arrVertices[5] == pQuad->vertex( 1 )) &&
                (arrVertices[6] == pQuad->vertex( 2 )) &&
                (arrVertices[7] == pQuad->vertex( 3 ))) )
            cout << "T3DQuad::checkNeighbours : wrong up neighbour" << endl;

        if ( ! ((arrEdges[4] == pQuad->edge( 7 ))  &&
                (arrEdges[9] == pQuad->edge( 2 ))  &&
                (arrEdges[11] == pQuad->edge( 8 ))  &&
                (arrEdges[13] == pQuad->edge( 15 ))  &&
                (arrEdges[14] == pQuad->edge( 17 ))) )
            cout << "T3DQuad::checkNeighbours : wrong up edges" << endl;
    }// if
        
    if (arrNeighbours[DOWN] != NULL)
    {
        pQuad = arrNeighbours[DOWN];
        
        if ( ! ((arrVertices[0] == pQuad->vertex( 4 )) &&
                (arrVertices[1] == pQuad->vertex( 5 )) &&
                (arrVertices[2] == pQuad->vertex( 6 )) &&
                (arrVertices[3] == pQuad->vertex( 7 ))) )
            cout << "T3DQuad::checkNeighbours : wrong down neighbour" << endl;

        if ( ! ((arrEdges[2] == pQuad->edge( 9 ))  &&
                (arrEdges[7] == pQuad->edge( 4 ))  &&
                (arrEdges[8] == pQuad->edge( 11 ))  &&
                (arrEdges[15] == pQuad->edge( 13 ))  &&
                (arrEdges[17] == pQuad->edge( 14 )) ) )
            cout << "T3DQuad::checkNeighbours : wrong down edges" << endl;
    }// if
}

TVertex<3> *
T3DQuad::vertex ( uint i )
{
    return arrVertices[i];
}

T3DCellEdge *
T3DQuad::edge ( uint i )
{
    return arrEdges[i];
}

T3DCellEdge *
T3DQuad::edge ( uint i, uint j )
{
    //
    // access by vertex-indices (see cube.tri)
    //
    switch (i)
    {
        case 0:
            switch (j)
            {
                case 1: return arrEdges[2];  break;
                case 2: return arrEdges[8];  break;
                case 3: return arrEdges[17]; break;
                case 4: return arrEdges[10]; break;
                case 5: return arrEdges[0];  break;
                case 6: return arrEdges[5];  break;
                case 7: return arrEdges[12]; break;
                default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
            }// switch
            break;
            
        case 1:
            switch (j)
            {
                case 0: return arrEdges[0];  break;
                case 2: return arrEdges[7];  break;
                case 5: return arrEdges[1];  break;
                case 6: return arrEdges[3];  break;
                default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
            }// switch
            break;
            
        case 2:
            switch (j)
            {
                case 0: return arrEdges[8];  break;
                case 1: return arrEdges[7];  break;
                case 3: return arrEdges[15]; break;
                case 6: return arrEdges[6];  break;
                default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
            }// switch
            break;
            
        case 3:
            switch (j)
            {
                case 0: return arrEdges[17]; break;
                case 2: return arrEdges[15]; break;
                case 6: return arrEdges[16]; break;
                case 7: return arrEdges[18]; break;
                default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
            }// switch
            break;
            
        case 4:
            switch (j)
            {
                case 0: return arrEdges[10]; break;
                case 5: return arrEdges[9];  break;
                case 6: return arrEdges[11]; break;
                case 7: return arrEdges[14]; break;
                default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
            }// switch
            break;
            
        case 5:
            switch (j)
            {
                case 0: return arrEdges[0];  break;
                case 1: return arrEdges[1];  break;
                case 4: return arrEdges[9];  break;
                case 6: return arrEdges[4];  break;
                default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
            }// switch
            break;
            
        case 6:
            switch (j)
            {
                case 0: return arrEdges[5];  break;
                case 1: return arrEdges[3];  break;
                case 2: return arrEdges[6];  break;
                case 3: return arrEdges[16]; break;
                case 4: return arrEdges[11]; break;
                case 5: return arrEdges[4];  break;
                case 7: return arrEdges[13]; break;
                default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
            }// switch
            break;
            
        case 7:
            switch (j)
            {
                case 0: return arrEdges[12]; break;
                case 3: return arrEdges[18]; break;
                case 4: return arrEdges[14]; break;
                case 6: return arrEdges[13]; break;
                default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
            }// switch
            break;

        default : cout << "T3DQuad::edge (...) : wrong index" << endl; break;
    }// switch

    return NULL;
}

T3DFrontier *
T3DQuad::frontier ( uint i )
{
    return arrFrontiers[i];
}

T3DFrontier *
T3DQuad::frontier ( uint i, uint j, uint k )
{
    //
    // access by vertex-indices
    //
    return NULL;
}

///////////////////////////////////////////
//
// build/handle net-elements
//

bool
T3DQuad::buildVertex ( uint i, TFloat x, TFloat y, TFloat z )
{
    if (i > 7)
    {
        cout << "T3DQuad::buildVertex (...) : wrong index" << endl;
        return FALSE;
    }// if

    TVertex<3>  * pVertex = NULL;
    T3DQuad     * pAdj;
    
    if (arrVertices[i] != NULL)
        return FALSE;

    //
    // look if a neighbour has already built vertex
    //
    if (i == 0)
    {
        if (((pAdj = neighbour( LEFT )) != NULL) && ((pVertex = pAdj->vertex( 1 )) != NULL))
            arrVertices[0] = pVertex;
        else if (((pAdj = neighbour( FRONT )) != NULL) && ((pVertex = pAdj->vertex( 3 )) != NULL))
            arrVertices[0] = pVertex;
        else if (((pAdj = neighbour( DOWN )) != NULL) && ((pVertex = pAdj->vertex( 4 )) != NULL))
            arrVertices[0] = pVertex;
    }// if
    else if (i == 1)
    {
        if (((pAdj = neighbour( RIGHT )) != NULL) && ((pVertex = pAdj->vertex( 0 )) != NULL))
            arrVertices[1] = pVertex;
        else if (((pAdj = neighbour( FRONT )) != NULL) && ((pVertex = pAdj->vertex( 2 )) != NULL))
            arrVertices[1] = pVertex;
        else if (((pAdj = neighbour( DOWN )) != NULL) && ((pVertex = pAdj->vertex( 5 )) != NULL))
            arrVertices[1] = pVertex;
    }// else if
    else if (i == 2)
    {
        if (((pAdj = neighbour( RIGHT )) != NULL) && ((pVertex = pAdj->vertex( 3 )) != NULL))
            arrVertices[2] = pVertex;
        else if (((pAdj = neighbour( BACK )) != NULL) && ((pVertex = pAdj->vertex( 1 )) != NULL))
            arrVertices[2] = pVertex;
        else if (((pAdj = neighbour( DOWN )) != NULL) && ((pVertex = pAdj->vertex( 6 )) != NULL))
            arrVertices[2] = pVertex;
    }// else if
    else if (i == 3)
    {
        if (((pAdj = neighbour( LEFT )) != NULL) && ((pVertex = pAdj->vertex( 2 )) != NULL))
            arrVertices[3] = pVertex;
        else if (((pAdj = neighbour( BACK )) != NULL) && ((pVertex = pAdj->vertex( 0 )) != NULL))
            arrVertices[3] = pVertex;
        else if (((pAdj = neighbour( DOWN )) != NULL) && ((pVertex = pAdj->vertex( 7 )) != NULL))
            arrVertices[3] = pVertex;
    }// else if
    else if (i == 4)
    {
        if (((pAdj = neighbour( LEFT )) != NULL) && ((pVertex = pAdj->vertex( 5 )) != NULL))
            arrVertices[4] = pVertex;
        else if (((pAdj = neighbour( FRONT )) != NULL) && ((pVertex = pAdj->vertex( 7 )) != NULL))
            arrVertices[4] = pVertex;
        else if (((pAdj = neighbour( UP )) != NULL) &&  ((pVertex = pAdj->vertex( 0 )) != NULL))
            arrVertices[4] = pVertex;
    }// else if
    else if (i == 5)
    {
        if (((pAdj = neighbour( RIGHT )) != NULL) && ((pVertex = pAdj->vertex( 4 )) != NULL))
            arrVertices[5] = pVertex;
        else if (((pAdj = neighbour( FRONT )) != NULL) && ((pVertex = pAdj->vertex( 6 )) != NULL))
            arrVertices[5] = pVertex;
        else if (((pAdj = neighbour( UP )) != NULL) && ((pVertex = pAdj->vertex( 1 )) != NULL))
            arrVertices[5] = pVertex;
    }// else if
    else if (i == 6)
    {
        if (((pAdj = neighbour( RIGHT )) != NULL) && ((pVertex = pAdj->vertex( 7 )) != NULL))
            arrVertices[6] = pVertex;
        else if (((pAdj = neighbour( BACK )) != NULL) && ((pVertex = pAdj->vertex( 5 )) != NULL))
            arrVertices[6] = pVertex;
        else if (((pAdj = neighbour( UP )) != NULL) &&  ((pVertex = pAdj->vertex( 2 )) != NULL))
            arrVertices[6] = pVertex;
    }// else if
    else if (i == 7)
    {
        if (((pAdj = neighbour( LEFT )) != NULL) && ((pVertex = pAdj->vertex( 6 )) != NULL))
            arrVertices[7] = pVertex;
        else if (((pAdj = neighbour( BACK )) != NULL) && ((pVertex = pAdj->vertex( 4 )) != NULL))
            arrVertices[7] = pVertex;
        else if (((pAdj = neighbour( UP )) != NULL) && ((pVertex = pAdj->vertex( 3 )) != NULL))
            arrVertices[7] = pVertex;
    }// else if

    if (pVertex == NULL)
    {
        pVertex = new TVertex<3>( x, y, z, pDomain->boundary( x, y, z ) );
        pVertex->delRef();
        arrVertices[i] = pVertex;
        return TRUE;
    }// if
    else
        return FALSE;
}

void
T3DQuad::buildEdge ( uint i )
{
    if (i > 18)
    {
        cout << "T3DQuad::buildEdge (...) : wrong index" << endl;
        return;
    }// if

    T3DCellEdge  * pEdge = NULL;
    T3DQuad      * pAdj;
    
    if (arrEdges[i] != NULL)
        return;

    //
    // look if a neighbour has already built edge
    //
    switch (i)
    {
        case 0:
            if (((pAdj = neighbour( FRONT )) != NULL) && ((pEdge = pAdj->edge( 16 )) != NULL))
                arrEdges[0] = pEdge;
            break;
            
        case 1:
            if (((pAdj = neighbour( FRONT )) != NULL) && ((pEdge = pAdj->edge( 6 )) != NULL))
                arrEdges[1] = pEdge;
            else if (((pAdj = neighbour( RIGHT )) != NULL) && ((pEdge = pAdj->edge( 10 )) != NULL))
                arrEdges[1] = pEdge;
            break;
            
        case 2:
            if (((pAdj = neighbour( FRONT )) != NULL) && ((pEdge = pAdj->edge( 15 )) != NULL))
                arrEdges[2] = pEdge;
            else if (((pAdj = neighbour( DOWN )) != NULL) && ((pEdge = pAdj->edge( 9 )) != NULL))
                arrEdges[2] = pEdge;
            break;
            
        case 3:
            if (((pAdj = neighbour( RIGHT )) != NULL) && ((pEdge = pAdj->edge( 12 )) != NULL))
                arrEdges[3] = pEdge;
            break;
            
        case 4:
            if (((pAdj = neighbour( RIGHT )) != NULL) && ((pEdge = pAdj->edge( 14 )) != NULL))
                arrEdges[4] = pEdge;
            else if (((pAdj = neighbour( UP )) != NULL) && ((pEdge = pAdj->edge( 7 )) != NULL))
                arrEdges[4] = pEdge;
            break;
            
        case 5:               // interior edge
            break;
            
        case 6:
            if (((pAdj = neighbour( RIGHT )) != NULL) && ((pEdge = pAdj->edge( 18 )) != NULL))
                arrEdges[6] = pEdge;
            else if (((pAdj = neighbour( BACK )) != NULL) && ((pEdge = pAdj->edge( 1 )) != NULL))
                arrEdges[6] = pEdge;
            break;
            
        case 7:
            if (((pAdj = neighbour( RIGHT )) != NULL) && ((pEdge = pAdj->edge( 17 )) != NULL))
                arrEdges[7] = pEdge;
            else if (((pAdj = neighbour( DOWN )) != NULL) &&  ((pEdge = pAdj->edge( 4 )) != NULL))
                arrEdges[7] = pEdge;
            break;
            
        case 8:
            if (((pAdj = neighbour( DOWN )) != NULL) && ((pEdge = pAdj->edge( 11 )) != NULL))
                arrEdges[8] = pEdge;
            break;
            
        case 9:
            if (((pAdj = neighbour( FRONT )) != NULL) && ((pEdge = pAdj->edge( 13 )) != NULL))
                arrEdges[9] = pEdge;
            else if (((pAdj = neighbour( UP )) != NULL) && ((pEdge = pAdj->edge( 2 )) != NULL))
                arrEdges[9] = pEdge;
            break;
            
        case 10:
            if (((pAdj = neighbour( LEFT )) != NULL) && ((pEdge = pAdj->edge( 1 )) != NULL))
                arrEdges[10] = pEdge;
            else if (((pAdj = neighbour( FRONT )) != NULL) && ((pEdge = pAdj->edge( 18 )) != NULL))
                arrEdges[10] = pEdge;
            break;
            
        case 11:
            if (((pAdj = neighbour( UP )) != NULL) && ((pEdge = pAdj->edge( 8 )) != NULL))
                arrEdges[11] = pEdge;
            break;
            
        case 12:
            if (((pAdj = neighbour( LEFT )) != NULL) && ((pEdge = pAdj->edge( 3 )) != NULL))
                arrEdges[12] = pEdge;
            break;
            
        case 13:
            if (((pAdj = neighbour( BACK )) != NULL) && ((pEdge = pAdj->edge( 9 )) != NULL))
                arrEdges[13] = pEdge;
            else if (((pAdj = neighbour( UP )) != NULL) && ((pEdge = pAdj->edge( 15 )) != NULL))
                arrEdges[13] = pEdge;
            break;
            
        case 14:
            if (((pAdj = neighbour( LEFT )) != NULL) && ((pEdge = pAdj->edge( 4 )) != NULL))
                arrEdges[14] = pEdge;
            else if (((pAdj = neighbour( UP )) != NULL) && ((pEdge = pAdj->edge( 17 )) != NULL))
                arrEdges[14] = pEdge;
            break;
            
        case 15:
            if (((pAdj = neighbour( BACK )) != NULL) && ((pEdge = pAdj->edge( 2 )) != NULL))
                arrEdges[15] = pEdge;
            else if (((pAdj = neighbour( DOWN )) != NULL) && ((pEdge = pAdj->edge( 13 )) != NULL))
                arrEdges[15] = pEdge;
            break;
            
        case 16:
            if (((pAdj = neighbour( BACK )) != NULL) && ((pEdge = pAdj->edge( 0 )) != NULL))
                arrEdges[16] = pEdge;
            break;
            
        case 17:
            if (((pAdj = neighbour( LEFT )) != NULL) && ((pEdge = pAdj->edge( 7 )) != NULL))
                arrEdges[17] = pEdge;
            else if (((pAdj = neighbour( DOWN )) != NULL) && ((pEdge = pAdj->edge( 14 )) != NULL))
                arrEdges[17] = pEdge;
            break;
            
        case 18:
            if (((pAdj = neighbour( LEFT )) != NULL) && ((pEdge = pAdj->edge( 6 )) != NULL))
                arrEdges[18] = pEdge;
            else if (((pAdj = neighbour( BACK )) != NULL) && ((pEdge = pAdj->edge( 10 )) != NULL))
                arrEdges[18] = pEdge;
            break;
    }// switch

    if (pEdge == NULL)
    {
        TVector<3>   v;
        uint         i1 = 0, i2 = 0;

        switch (i)
        {
            case  0: i1 = 0; i2 = 5; break;
            case  1: i1 = 5; i2 = 1; break;
            case  2: i1 = 1; i2 = 0; break;
            case  3: i1 = 1; i2 = 6; break;
            case  4: i1 = 6; i2 = 5; break;
            case  5: i1 = 6; i2 = 0; break;
            case  6: i1 = 6; i2 = 2; break;
            case  7: i1 = 2; i2 = 1; break;
            case  8: i1 = 2; i2 = 0; break;
            case  9: i1 = 5; i2 = 4; break;
            case 10: i1 = 4; i2 = 0; break;
            case 11: i1 = 4; i2 = 6; break;
            case 12: i1 = 0; i2 = 7; break;
            case 13: i1 = 7; i2 = 6; break;
            case 14: i1 = 7; i2 = 4; break;
            case 15: i1 = 2; i2 = 3; break;
            case 16: i1 = 3; i2 = 6; break;
            case 17: i1 = 0; i2 = 3; break;
            case 18: i1 = 7; i2 = 3; break;
        }// switch

        // get boundary at edge midpoint
        v  = * arrVertices[i1];
        v += * arrVertices[i2];
        v *= 0.5;

        pEdge = new T3DCellEdge( arrVertices[i1], arrVertices[i2],
                                 pDomain->boundary( & v ) );
        
        pEdge->delRef();
        arrEdges[i] = pEdge;
    }// if
}

void
T3DQuad::buildFrontier ( uint i )
{
    if (i > 17)
    {
        cout << "T3DQuad::buildFrontier (...) : wrong index" << endl;
        return;
    }// if

    T3DFrontier  * pFrontier = NULL;
    T3DQuad      * pAdj;
    
    if (arrFrontiers[i] != NULL)
        return;

    //
    // look if a neighbour has already built frontier
    //
    switch (i)
    {
        case  0:
            if (((pAdj = neighbour( FRONT )) != NULL) && ((pFrontier = pAdj->frontier( 13 )) != NULL))
                arrFrontiers[0] = pFrontier;
            break;
            
        case  1:
            if (((pAdj = neighbour( RIGHT )) != NULL) && ((pFrontier = pAdj->frontier( 11 )) != NULL))
                arrFrontiers[1] = pFrontier;
            break;
            
        case  4:
            if (((pAdj = neighbour( RIGHT )) != NULL) && ((pFrontier = pAdj->frontier( 17 )) != NULL))
                arrFrontiers[4] = pFrontier;
            break;
            
        case  6:
            if (((pAdj = neighbour( DOWN )) != NULL) && ((pFrontier = pAdj->frontier( 9 )) != NULL))
                arrFrontiers[6] = pFrontier;
            break;
            
        case  7:
            if (((pAdj = neighbour( FRONT )) != NULL) && ((pFrontier = pAdj->frontier( 16 )) != NULL))
                arrFrontiers[7] = pFrontier;
            break;
            
        case  9:
            if (((pAdj = neighbour( UP )) != NULL) && ((pFrontier = pAdj->frontier( 6 )) != NULL))
                arrFrontiers[9] = pFrontier;
            break;
            
        case 11:
            if (((pAdj = neighbour( LEFT )) != NULL) && ((pFrontier = pAdj->frontier( 1 )) != NULL))
                arrFrontiers[11] = pFrontier;
            break;
            
        case 12:
            if (((pAdj = neighbour( UP )) != NULL) && ((pFrontier = pAdj->frontier( 15 )) != NULL))
                arrFrontiers[12] = pFrontier;
            break;
            
        case 13:
            if (((pAdj = neighbour( BACK )) != NULL) && ((pFrontier = pAdj->frontier( 0 )) != NULL))
                arrFrontiers[13] = pFrontier;
            break;
            
        case 15:
            if (((pAdj = neighbour( DOWN )) != NULL) && ((pFrontier = pAdj->frontier( 12 )) != NULL))
                arrFrontiers[15] = pFrontier;
            break;
            
        case 16:
            if (((pAdj = neighbour( BACK )) != NULL) && ((pFrontier = pAdj->frontier( 7 )) != NULL))
                arrFrontiers[16] = pFrontier;
            break;
            
        case 17:
            if (((pAdj = neighbour( LEFT )) != NULL) && ((pFrontier = pAdj->frontier( 4 )) != NULL))
                arrFrontiers[17] = pFrontier;
            break;
    }// switch

    if (pFrontier == NULL)
    {
        uint         i1, i2, i3;
        TVector<3>   v;
       
        i1 = i2 = i3 = 0;
 
        switch (i)
        {
            case  0:  i1 = 0;  i2 = 1;  i3 = 2;   break;
            case  1:  i1 = 1;  i2 = 3;  i3 = 4;   break;
            case  2:  i1 = 3;  i2 = 5;  i3 = 2;   break;
            case  3:  i1 = 5;  i2 = 0;  i3 = 4;   break;
            case  4:  i1 = 3;  i2 = 6;  i3 = 7;   break;
            case  5:  i1 = 6;  i2 = 8;  i3 = 5;   break;
            case  6:  i1 = 2;  i2 = 7;  i3 = 8;   break;
            case  7:  i1 = 0;  i2 = 9;  i3 = 10;  break;
            case  8:  i1 = 5;  i2 = 10; i3 = 11;  break;
            case  9:  i1 = 9;  i2 = 11; i3 = 4;   break;
            case 10:  i1 = 5;  i2 = 12; i3 = 13;  break;
            case 11:  i1 = 10; i2 = 12; i3 = 14;  break;
            case 12:  i1 = 11; i2 = 13; i3 = 14;  break;
            case 13:  i1 = 6;  i2 = 15; i3 = 16;  break;
            case 14:  i1 = 5;  i2 = 17; i3 = 16;  break;
            case 15:  i1 = 17; i2 = 15; i3 = 8;   break;
            case 16:  i1 = 18; i2 = 16; i3 = 13;  break;
            case 17:  i1 = 12; i2 = 18; i3 = 17;  break;
        }// switch

        pFrontier = new T3DTriangle( NULL, arrEdges[i1], arrEdges[i2], arrEdges[i3] );
        pFrontier->delRef();
        arrFrontiers[i] = pFrontier;
        
        //
        // get boundary at frontier midpoint
        //
        v  = * pFrontier->getNode( 0 );
        v += * pFrontier->getNode( 2 );
        v += * pFrontier->getNode( 4 );

        v *= 1.0 / 3.0;

        pFrontier->setBoundary( pDomain->boundary( & v ) );
    }// if
}

void
T3DQuad::buildCell ( uint i )
{
    if (i > 5)
    {
        cout << "T3DQuad::buildCell (...) : wrong index" << endl;
        return;
    }// if

    TCell<3>  * pCell = NULL;
    
    if (arrCells[i] != NULL)
        return;

    switch (i)
    {
        case 0:
            pCell = new T3DTetrahedron( NULL,
                                        (T3DTriangle*) arrFrontiers[0],
                                        (T3DTriangle*) arrFrontiers[1],
                                        (T3DTriangle*) arrFrontiers[2],
                                        (T3DTriangle*) arrFrontiers[3] );
            break;
            
        case 1:
            pCell = new T3DTetrahedron( NULL,
                                        (T3DTriangle*) arrFrontiers[2],
                                        (T3DTriangle*) arrFrontiers[4],
                                        (T3DTriangle*) arrFrontiers[5],
                                        (T3DTriangle*) arrFrontiers[6] );
            break;
            
        case 2:
            pCell = new T3DTetrahedron( NULL,
                                        (T3DTriangle*) arrFrontiers[7],
                                        (T3DTriangle*) arrFrontiers[3],
                                        (T3DTriangle*) arrFrontiers[8],
                                        (T3DTriangle*) arrFrontiers[9] );
            break;
            
        case 3:
            pCell = new T3DTetrahedron( NULL,
                                        (T3DTriangle*) arrFrontiers[8],
                                        (T3DTriangle*) arrFrontiers[10],
                                        (T3DTriangle*) arrFrontiers[11],
                                        (T3DTriangle*) arrFrontiers[12] );
            break;
            
        case 4:
            pCell = new T3DTetrahedron( NULL,
                                        (T3DTriangle*) arrFrontiers[5],
                                        (T3DTriangle*) arrFrontiers[13],
                                        (T3DTriangle*) arrFrontiers[14],
                                        (T3DTriangle*) arrFrontiers[15] );
            break;
            
        case 5:
            pCell = new T3DTetrahedron( NULL,
                                        (T3DTriangle*) arrFrontiers[10],
                                        (T3DTriangle*) arrFrontiers[14],
                                        (T3DTriangle*) arrFrontiers[16],
                                        (T3DTriangle*) arrFrontiers[17] );
            break;
    }// switch

    pCell->delRef();
    arrCells[i] = pCell;
}

void
T3DQuad::buildAllEdges ()
{
    uint  i;
    
    for ( i = 0; i < 19; i++ )
        buildEdge( i );
}

void
T3DQuad::buildAllFrontiers ()
{
    uint  i;
    
    for ( i = 0; i < 18; i++ )
        buildFrontier( i );
}

void
T3DQuad::buildAllCells ()
{
    uint  i;

    for ( i = 0; i < 6; i++ )
    {
        buildCell( i );
    }// for
}

void
T3DQuad::addCells ( TNet<3> * pNet )
{
    if (pNet == NULL)
    {
        cout << "T3DQuad::addCells (...) : got NULL arg" << endl;
        return;
    }// if

    uint       i;
    TCell<3> * pCell;

    for ( i = 0; i < 6; i++ )
    {
        pCell = arrCells[i];
        
        if (pCell != NULL)
            pNet->add( pCell );
        else
            cout << "T3DQuad::addCells (...) : have NULL cell" << endl;
    }// for
}

///////////////////////////////////////////
///////////////////////////////////////////
//
// class for a simple meshgenerator in 3D
//
///////////////////////////////////////////
///////////////////////////////////////////

///////////////////////////////////////////
//
// constructors and destructor
//

T3DMeshGenerator::T3DMeshGenerator ()
{}

T3DMeshGenerator::~T3DMeshGenerator ()
{
    TListIterator< T3DQuad >   iter;
    T3DQuad                  * pQuad;

    iter.setList( listQuads );
    iter.first();
    while ((pQuad = iter++) != NULL)
    {
        delete pQuad;
    }// while
}

///////////////////////////////////////////
//
// access member variables
//

TList< T3DQuad > &
T3DMeshGenerator::quads ()
{
    return listQuads;
}
    
///////////////////////////////////////////
//
// build net of quads
//

TNet<3> *
T3DMeshGenerator::buildNet ()
{
    TListIterator< T3DQuad >   iter;
    T3DQuad                  * pQuad;
    TNet<3>                  * pNet = new TNet<3>;

    iter.setList( listQuads );

    iter.first();
    while ((pQuad = iter++) != NULL)
    {
        pQuad->setNeighbours( & listQuads );
    }// while
    
//     iter.first();
//     while ((pQuad = iter++) != NULL)
//     {
//         pQuad->checkNeighbours();
//     }// while
    
    iter.first();
    while ((pQuad = iter++) != NULL)
    {
        pQuad->buildAllEdges();
        pQuad->buildAllFrontiers();
        pQuad->buildAllCells();

        pQuad->addCells( pNet );
    }// while

    return pNet;
}

TNet<3> *
T3DMeshGenerator::buildNet ( uint sizeX, TFloat arrCoordX[],
                             uint sizeY, TFloat arrCoordY[],
                             uint sizeZ, TFloat arrCoordZ[],
                             TDomain * pDomain )
{
    uint       i, j, k;
    T3DQuad  * pQuad;
    TNet<3>  * pNet = new TNet<3>;

    T3DQuad  * arrQuads[sizeX][sizeY][sizeZ];
    
    //
    // init. quad-array
    //

    for ( i = 0; i < sizeX - 1; i++ )
        for ( j = 0; j < sizeY - 1; j++ )
            for ( k = 0; k < sizeZ - 1; k++ )
                arrQuads[i][j][k] = NULL;

    //
    // create quads
    //
    for ( k = 0; k < sizeZ - 1; k++ )
    {
        for ( j = 0; j < sizeY - 1; j++ )
        {
            for ( i = 0; i < sizeX - 1; i++ )
            {
                pQuad = new T3DQuad( pDomain );

                addQuad( pQuad );
                
                arrQuads[i][j][k] = pQuad;

                //
                // build connectivity
                //
                if (i > 0)        pQuad->setNeighbour( LEFT,  arrQuads[i-1][j][k] );
                if (i < sizeX-2)  pQuad->setNeighbour( RIGHT, arrQuads[i+1][j][k] );

                if (j > 0)        pQuad->setNeighbour( FRONT, arrQuads[i][j-1][k] );
                if (j < sizeY-2)  pQuad->setNeighbour( BACK,  arrQuads[i][j+1][k] );

                if (k > 0)        pQuad->setNeighbour( DOWN,  arrQuads[i][j][k-1] );
                if (k < sizeZ-2)  pQuad->setNeighbour( UP,    arrQuads[i][j][k+1] );

                //
                // build vertices, edges and frontiers
                //
                pQuad->buildVertex( 0, arrCoordX[i],   arrCoordY[j],   arrCoordZ[k]   );
                pQuad->buildVertex( 1, arrCoordX[i+1], arrCoordY[j],   arrCoordZ[k]   );
                pQuad->buildVertex( 2, arrCoordX[i+1], arrCoordY[j+1], arrCoordZ[k]   );
                pQuad->buildVertex( 3, arrCoordX[i],   arrCoordY[j+1], arrCoordZ[k]   );
                pQuad->buildVertex( 4, arrCoordX[i],   arrCoordY[j],   arrCoordZ[k+1] );
                pQuad->buildVertex( 5, arrCoordX[i+1], arrCoordY[j],   arrCoordZ[k+1] );
                pQuad->buildVertex( 6, arrCoordX[i+1], arrCoordY[j+1], arrCoordZ[k+1] );
                pQuad->buildVertex( 7, arrCoordX[i],   arrCoordY[j+1], arrCoordZ[k+1] );

                pQuad->buildAllEdges();
                
                pQuad->buildAllFrontiers();

                pQuad->buildAllCells();

                //
                // add all cells to net
                //
                pQuad->addCells( pNet );
            }// for
        }// for
    }// for

    return pNet;
}

void
T3DMeshGenerator::addQuad ( T3DQuad * quad )
{
    listQuads.add( * quad );
}
