//
//  Project   : libnet
//  File      : TBoundaryDesc.cc
//  Author    : Ronald Kriemann
//  Purpose   : class for representing info about boundaryvalues
//
// arch-tag: 8a1e1aea-40b4-4a3c-8f98-dfb9e6faf6bc
//

#include "TBoundaryDesc.hh"

//
// constructor
//
TBoundaryDesc::TBoundaryDesc ( int n )
{
    setNoOfTypes( n );
    clearFlags();
}

//
// destructor
//
TBoundaryDesc::~TBoundaryDesc ()
{
}

//
// set (get) number of types to use
//
uint
TBoundaryDesc::getNoOfTypes () const
{
    return arrType.getSize();
}

void
TBoundaryDesc::setNoOfTypes ( uint n )
{
    uint  oldSize = arrType.getSize();
    uint  i;
    
    arrType.setSize( n );

    clearFlags();

    for (i = oldSize; i < n; i++)
    {
        arrType[i] = NO_TYPE;
    }// for
}
    
//
// access type info
//
const int
TBoundaryDesc::getType ( uint i ) const
{
    if (i < getNoOfTypes())
        return arrType[i];
    else
    {
        cout << "TBoundaryDesc::getType (" << i << ") : index too high" << endl;
        return NO_TYPE;
    }// else
}

void
TBoundaryDesc::setType ( uint i, int t )
{
    if (i < getNoOfTypes())
        arrType[i] = t;
    else
        cout << "TBoundaryDesc::setType (" << i << ",int) : index too high" << endl;
}

//
// access DoF info
//
const bool
TBoundaryDesc::isDoF ( uint i ) const
{
    if (i < getNoOfTypes())
    {
        return hasFlag( 1 << i );
    }// if
    else
    {
        cout << "TBoundaryDesc::isDoF (" << i << ") : index too high" << endl;
        return FALSE;
    }// else
}

void
TBoundaryDesc::setDoFInfo ( uint i, bool b )
{
    if (i < getNoOfTypes())
    {
        if (b == TRUE)
            setFlag( 1 << i );
        else
            clearFlag( 1 << i );
    }// if
    else
        cout << "TBoundaryDesc::setDoFInfo (" << i << ") : index too high" << endl;
}
 
void
TBoundaryDesc::setDoFInfo ()
{
    uint  i;
    uint  size = getNoOfTypes();
    
    for ( i = 0; i < size; i++ )
    {
        if (arrType[i] == NEU_TYPE)
            setDoFInfo( i, TRUE );
        else
        {
            if (arrType[i] == DIR_TYPE)
                setDoFInfo( i, FALSE );
            else
                cerr << "TBoundaryDesc::setDoFInfo () :  i" << i
                     << " no such Type:" << arrType[i]<< endl;
        }// else
    }// for
} // setDoFInfo

int 
TBoundaryDesc::getN() const
{
    cerr << "TBoundaryDesc::getN() : carries no information about Dimension " << endl;
    
    return 0;
}
