//
//  Project  : libmisc
//  File     : parser.cc
//  Author   : Ronald Kriemann
//  Purpose  : class for parsing a file for the init. tri.
//
// arch-tag: 96566f3c-965c-4df6-9688-33367fb0efc7
//

#include <stdio.h>

#include "parser.hh"

// decl for funtions in parser-file
extern int 
parseFile( FILE                * pFile,
           TList< TCell<2> >   * pList2D,
           TList< TCell<3> >   * pList3D );

//
// constructor
//
TParser::TParser ( const char *filename ) 
{
    strcpy( szFilename, filename );

    pList2DCells = NULL;
    pList3DCells = NULL;
}

//
// build the celllist with the init. tri.
//
TParser &
TParser::buildList () 
{
    FILE  * pFile;
    
    pFile = fopen( szFilename, "r" );
    if (pFile == NULL)
    {
        cerr << "TParser::buildList () : could not open file : " << szFilename << endl;
        exit( 1 );
    }// if
    else
    {
        if (pList2DCells != NULL)
            delete pList2DCells;
        
        if (pList3DCells != NULL)
            delete pList3DCells;
        
        pList2DCells = new TList< TCell<2> >;
        pList3DCells = new TList< TCell<3> >;
        
        parseFile( pFile, pList2DCells, pList3DCells );

        if (pList2DCells->getSize() == 0)
        {
            delete pList2DCells;
            pList2DCells = NULL;
        }// if

        if (pList3DCells->getSize() == 0)
        {
            delete pList3DCells;
            pList3DCells = NULL;
        }// if
    }// else

    return *this;
}

//
// access cell-lists
//
TList< TCell<2> > *
TParser::get2DCellList () const
{
    return pList2DCells;
}

TList< TCell<3> > *
TParser::get3DCellList () const
{
    return pList3DCells;
}

void *
TParser::getCellList ( int n ) const
{
    if (n == 2)
        return pList2DCells;

    if (n == 3)
        return pList3DCells;

    return NULL;
}

//
// return dim of red file (0 for no list built)
//
int
TParser::getDim () const
{
    if (pList2DCells != NULL)
        return 2;

    if (pList3DCells != NULL)
        return 3;

    return 0;
}
