//
//  Project   : libnet
//  File      : TPrecSolver.cc
//  Author    : Ronald Kriemann
//  Purpose   : baseclass for solver-classes using a preconditioner
//
// arch-tag: de7db5ae-62e4-4ead-a22b-c8b962c09f9f
//

extern "C" {
#include <stdio.h>
}

#include "TPrecSolver.hh"

//
// constructor
//
TPrecSolver::TPrecSolver( TSolver              * prec,
                          TBlockMatrix         * pA, 
                          TBlockVector<TFloat> * pRHS, 
                          TBlockVector<TFloat> * pX, 
                          uint                   maxiter, 
                          TFloat                 accur, 
                          uint                   verb )
        : TSolver( pA, pRHS, pX, maxiter, accur, verb )
{
    setPrec( prec );
}

//
// access precond.
//

TSolver *
TPrecSolver::getPrec () const
{
    return pPrecond;
}

void
TPrecSolver::setPrec ( TSolver * precond )
{
    if (precond == NULL)
    {
        cout << "TPrecSolver::setPrec : invalid preconditioner (NULL)" << endl;
        exit( 1 );
    }// if
    
	pPrecond  = precond;

    if (dynamic_cast< TSecondNF * >( precond ) != NULL)
        bSecondNF = TRUE;
    else
        bSecondNF = FALSE;

    cleanName();
    setName( getName() );
}

const bool
TPrecSolver::isPrecSecondNF () const
{
    return bSecondNF;
}

//
// overload naming
//

void
TPrecSolver::cleanName ()
{
    TString  tmpStr, cleanStr;

    tmpStr = getName().strchr( '(' );
    
    if (tmpStr.length() > 0)
    {
        cleanStr = getName().substr( 0, getName().length() - tmpStr.length() - 1 );
        TSolver::setName( cleanStr );
    }// if
}

void
TPrecSolver::setName ( const TString & n )
{
	if (pPrecond != NULL)
	{
        TString  newName;

        newName  = n;
        newName += " (";
        newName += pPrecond->getName();
        newName += ")";

        TSolver::setName( newName );
	}// if
    else
        TSolver::setName( n );
}
