//
//  Project   : libnet
//  File      : fixedPointDefectCorrection.cc
//  Author    : Sabine Gutsch / Thomas Probst
//  Purpose   : implementation of a nonlinear solver
//
// arch-tag: d38b2c7d-0d29-4d88-9f83-df51361c9667
//

#include "fixedPointDefectCorrection.hh"

// constructor
TFixedPointDefectCorrection::TFixedPointDefectCorrection ( TBasisFE<2> *paFE, TNet<2> *paNet, 
														   TSolver *pS, TBlockMatrix *paA, TBlockVector<TFloat> *paRHS, 
														   uint amaxit) 
	: TNonLinearSolver ( paFE, paNet, pS, paA, paRHS, amaxit) 
{
	setName("Fixed point defect correction");
}

void 
TFixedPointDefectCorrection::updateLinearProblem () 
{ 
	// first update matrix pA
	
	// subtract 'old' nonsymmetric part from diagonal blocks
	for (uint i; i < ( pA->getNoOfRows() - 1 ); i++)
		(*pA)[i][i] -= (*pN);
	
	// calculate 'new' nonsymmetric part
	pFE->discr_N ( pNet, pN );
	
	// add 'new' nonsymmetric part to diagonal blocks
	for (uint i; i < ( pA->getNoOfRows() - 1 ); i++)
		(*pA)[i][i] += (*pN);
	
	// now update rhs
	TBlockVector<TFloat> *pr = pLinearSolver->getRHS();
	(*pr) = (*pRHS) - (*pA) * (*x);
}
