//
//  Project  : libnet
//  File     : optSquaredSmoother2.cc
//  Author   : Sabine Gutsch 
//  Purpose  : smoother for a general system
//
// arch-tag: b02afe73-e5e4-4831-9a31-6a53747c8257
//

#include "optSquaredSmoother2.hh"

//
// constructor
//
TOptSquaredSmoother2::TOptSquaredSmoother2 ( TBlockMatrix *paA = NULL, 
                 TBlockVector<TFloat> *paRHS = NULL, 
				 TBlockVector<TFloat> *paX = NULL, 
				 uint amaxiter = DEFAULT_MAX_ITERATION,
				 TFloat aaccur = DEFAULT_ACCURACY, 
				 uint averb = 0 )
        : TSolver ( paA, paRHS, paX, amaxiter, aaccur, averb ) 
{ 
	setName("Optimal squared smoother 2");
}
 
//
// initialize 
//
void 
TOptSquaredSmoother2::init ()
{
    lambda = (*pA)[0][0].rowSumNorm() / 
             ( (*pA)[0][2].rowSumNorm() + 2 * (*pA)[2][0].rowSumNorm() );
    //lambda *= lambda;
	//cout << "\nlambda: " << lambda << endl;
	r  = (*pA) * (*pX); 
	r -= (*pRHS);        // r = Ax - b
    r[2] *= lambda;
}

//
// one iteration step
//
int
TOptSquaredSmoother2::step ( )
{    
    p = r;
    p[2] *= lambda; 
    p  = pA->transposeMult( p );
    p[2] *= lambda;

    ap = p;
    ap[2] *= lambda;
    ap = (*pA) * ap;
    ap[2] *= lambda;

    sigma = (p * p) / (ap * ap);
    p[2] *= lambda;
    p    *= sigma;
    ap   *= sigma;

    (*pX) -= p; 
    r     -= ap;

	// pressure correction
	TVectorBlock<TFloat> vecOne( r[2] );
    vecOne.set( 1.0 );
    TFloat factor = (vecOne * (*pX)[2]) / (vecOne * vecOne);
	//cout << "factor: " << factor << endl;
    vecOne.set( factor );
    (*pX)[2] -= vecOne;

    return (RETURN_CONTINUE);
}
