//
//  Project  : libnet
//  File     : residualSmoother.cc
//  Author   : Sabine Gutsch 
//  Purpose  : smoother for a general system
//
// arch-tag: e5427a55-4ce4-45b8-9782-e36bcfce6bc5
//

#include "residualSmoother.hh"

//
// constructor
//
TResidualSmoother::TResidualSmoother ( 
                 TSolver *pBasicIter,
                 TBlockMatrix *paA = NULL, 
                 TBlockVector<TFloat> *paRHS = NULL, 
				 TBlockVector<TFloat> *paX = NULL, 
				 uint amaxiter = DEFAULT_MAX_ITERATION,
				 TFloat aaccur = DEFAULT_ACCURACY, 
				 uint averb = 0 )
        : TSolver ( paA, paRHS, paX, amaxiter, aaccur, averb ) 
{ 
    pBasicIteration = pBasicIter;
    pBasicIteration->setX( pX );
    if (pBasicIteration != NULL)
	{
        TString newName;

        newName  = "ResSmoother ";
        newName += "(";
        newName += pBasicIteration->getName();
        newName += ")";
        setName( newName ); 
	}
    else
	    setName("Residual smoother");
}
 
//
// initialize 
//
void 
TResidualSmoother::init ()
{
    xt = (*pX);
    pBasicIteration->setX( &xt );
    pBasicIteration->setA( pA );
    pBasicIteration->setRHS( pRHS );
    pBasicIteration->init( );
    r = (*pRHS);
	r -= (*pA) * (*pX); 
    rt = r;
}

//
// one iteration step
//
int
TResidualSmoother::step ( )
{     
    pBasicIteration->step();
	rt = (*pRHS); 
	rt -= (*pA) * xt; 
    h = rt - r;
    eta = -(r * h) / (h * h);
    (*pX) *= (1-eta);
    (*pX) += (eta * xt);
    r *= (1-eta);
    r += (eta * rt); 
   
    return (RETURN_CONTINUE);
}
