//
//  Project   : Navier-Stokes
//  File      : printeps.cc
//  Author    : Ronald Kriemann
//  Purpose   : print convergence rates for different \epsilon
//              with logarithmic scaling
//
// arch-tag: 731161b2-cb96-4626-8b41-8ff1f8aee7bd
//

extern "C" {
#include <stdlib.h>
#include <string.h>
#include <math.h>
}

#include "defines.hh"

#include "printer.hh"

int
main ( int argc, char ** argv )
{
    char      filename[256], outfile[256], solver[256];
    FILE    * file;
    double  * val_x, * val_y;
    char      buffer[256];
    double    min_x, min_y, max_x, max_y;
    int       i, count;
    double    x, y;

    if (argc < 2)
    {
        cout << "main : expected filename" << endl;
        exit( 1 );
    }// if

    strcpy( filename, argv[1] );

    if (argc > 2)
        strcpy( solver, argv[2] );
    else
        strcpy( solver, "" );
        
    if ((file = fopen( filename, "r" )) == NULL)
    {
        cout << "main : can not open " << filename << endl;
        exit( 1 );
    }// if

    //
    // look how much entries we have
    //

    count = 0;
    while ( ! feof( file ) )
    {
        fgets( buffer, sizeof( buffer ), file );
        count++;
    }// while

    fseek( file, 0, SEEK_SET );
    
    //
    // read data in arrays
    //

    val_x = new double[ count ];
    val_y = new double[ count ];

    i = 0;
    while ( ! feof( file ) )
    {
        fgets( buffer, sizeof( buffer ), file );
        sscanf( buffer, "%lg %lg", & x, & y );

        val_x[i] = x;
        val_y[i] = y;
        
        i++;
    }// while

    fclose( file );

    //
    // look for minimal and maximal values
    //

    min_x = 1;
    min_y = 0;
    max_x = max_y = 1;

    for ( i = 0; i < count; i++ )
    {
        if (val_x[i] < min_x) min_x = val_x[i];
        if (val_x[i] > max_x) max_x = val_x[i];
        if (val_y[i] < min_y) min_y = val_y[i];
        if (val_y[i] > max_y) max_y = val_y[i];
    }// for

    //
    // draw values in logarithmic scale
    //

    strcpy( outfile,  strtok( filename, "." ) );
    strcat( outfile, ".eps" );

    TPrinter  printer( outfile );

    printer.getView().setWorld( log10( min_x )-0.7, -0.03, 0-log10( min_x )+0.7, 1.03 );

    printer.begin();
    
    printer.setRGBColor( COLOR_BLACK );

    for ( i = 1; i < count; i++ )
    {
        printer.drawLine( log10( val_x[i-1] ), val_y[i-1],
                          log10( val_x[i]   ), val_y[i]     );
    }// for

    // draw coord system
    printer.drawLine( log10( min_x ), 0, 0, 0 );

    for ( i = int(log10( min_x )); i <= 0; i++ )
    {
        printer.drawLine( i, 0, i, -0.01 );
        printer.setFont( "Courier", 14 );
        printer.drawText( i-0.3, -0.04, "10" );
        printer.setFont( "Courier", 14 );
        sprintf( buffer, " %d", i );
        printer.drawText( i-0.1, -0.03, buffer );
    }// for
    
    printer.drawLine( log10( min_x ), 0, log10( min_x ), 1 );

    printer.setFont( "Courier", 16 );
    for ( y = 0; y <= 1; y += 0.1 )
    {
        printer.drawLine( log10( min_x ), y, log10( min_x )-0.2, y );
        sprintf( buffer, "%.1f", y );
        printer.drawText( log10( min_x )-0.9, y-0.008, buffer );
    }// for

    // draw solvername
    if (strcmp( solver, "" ) != 0)
    {
        printer.setFont( "Times-Roman", 24 );
        printer.drawText( (log10( min_x ) / 2.0) - strlen( solver ) * 0.12, 1, solver );
    }// if
    
    printer.end();
    
    return 0;
}
