//
//  Project   : Navier-Stokes
//  File      : printquad.cc
//  Author    : Ronald Kriemann
//  Purpose   : print quad-grid
//
// arch-tag: 93c9ca85-9a7d-4e8a-9cf9-e6df14f81d66
//


#include "TVRML.hh"
#include "TPOVRay.hh"

#include "vector.hh"

void
printQuad( T3DPrinter & p )
{
    int     x, y, z;

    TFloat  cX[] = { 0, 0.1, 0.3, 1 };
    TFloat  cY[] = { 0, 0.1, 0.3, 1 };
    TFloat  cZ[] = { 0, 0.1, 0.3, 1 };

    int     nX, nY, nZ;
    TFloat  x_i, y_i, z_i;
    TFloat  x_ii, y_ii, z_ii;

    TVector<3>   v, u;

    nX = sizeof( cX ) / sizeof( TFloat );
    nY = sizeof( cY ) / sizeof( TFloat );
    nZ = sizeof( cZ ) / sizeof( TFloat );

    nZ = 2;
    
    p.begin();

    p.setRGBColor( COLOR_BLACK );
    
    for ( z = 0; z < nZ; z++ )
    {
        for ( y = 0; y < nY; y++ )
        {
            for ( x = 0; x < nX; x++ )
            {
                x_i = cX[x];
                y_i = cY[y];
                z_i = cZ[z];

                if (x < nX-1) x_ii = cX[x+1];
                if (y < nY-1) y_ii = cY[y+1];
                if (z < nZ-1) z_ii = cZ[z+1];

                if (x == 2 && y == 2) { x_i += 0.2;  y_i += 0.2; }
                if (x == 1 && y == 1) { x_ii += 0.2; y_ii += 0.2; }
                
                if (x < nX-1)
                {
                    v[X] = x_i ; v[Y] = y_i; v[Z] = z_i;
                    u[X] = x_ii; u[Y] = y_i; u[Z] = z_i;
                    p.drawLine( & v, & u );
                }// if

                if (y < nY-1)
                {
                    v[X] = x_i; v[Y] = y_i;  v[Z] = z_i;
                    u[X] = x_i; u[Y] = y_ii; u[Z] = z_i;
                    p.drawLine( & v, & u );
                }// if

                if (z < nZ-1)
                {
                    v[X] = x_i; v[Y] = y_i; v[Z] = z_i;
                    u[X] = x_i; u[Y] = y_i; u[Z] = z_ii;
                    p.drawLine( & v, & u );
                }// if
            }// for
        }// for
    }// for

//    p.drawCoordSystem();
    
    p.end();
}

int
main ( int, char ** )
{
    TVRML   vrml( "quad.wrl" );
    TPOVRay pov( "quad.pov" );

    printQuad( vrml );
    printQuad( pov );
}
