//
//  Project :  libvisual
//  File    :  viewport.cc
//  Purpose :  class for a world <-> viewport transformation
//  Author  :  Ronald Kriemann
//
// arch-tag: 903d4dca-9831-40d7-9ebc-c5d0fea7b818
//

#include "viewport.hh"

//
// constructor (does nothing)
//
TViewport::TViewport ()
{}

//
// constructor (gets all coord.)
//
TViewport::TViewport ( TFloat wXmin   , TFloat wYmin,
					   TFloat wXlength, TFloat wYlength,
					   TFloat vXmin   , TFloat vYmin,
					   TFloat vXlength, TFloat vYlength )
{
	viewXmin    = vXmin;
	viewYmin    = vYmin;;
	viewXlength = vXlength;
	viewYlength = vYlength;
	
	worldXmin    = wXmin;
	worldYmin    = wYmin;
	worldXlength = wXlength;
	worldYlength = wYlength;
	
	if (worldXlength != 0.0)
		scaleX = (TFloat)(viewXlength) / (worldXlength);
	else
		scaleX = 0.0;
	
	if (worldYlength != 0.0)
		scaleY = (TFloat)(viewYlength) / (worldYlength);
	else
		scaleY = 0.0;
}

//
// return the scalings
//
const TFloat 
TViewport::getScaleX () const
{
	return scaleX;
}

const TFloat 
TViewport::getScaleY () const
{
	return scaleY;
}

//
// set the worldcoord.
//
void 
TViewport::setWorld( TFloat wXmin   , TFloat wYmin,
					 TFloat wXlength, TFloat wYlength ) 
{
	worldXmin    = wXmin;
	worldYmin    = wYmin;
	worldXlength = wXlength;
	worldYlength = wYlength;
	
	if (worldXlength != 0.0)
		scaleX = (TFloat)(viewXlength) / (worldXlength);
	else
		scaleX = 0.0;
	
	if (worldYlength != 0.0)
		scaleY = (TFloat)(viewYlength) / (worldYlength);
	else
		scaleY = 0.0;
}

//
// get the worldborders
//
void 
TViewport::getWorld( TFloat& minX, TFloat& minY, TFloat& maxX, TFloat maxY ) const 
{
	minX = worldXmin;
	minY = worldYmin;
	maxX = worldXmin + worldXlength;
	maxY = worldYmin + worldYlength;
}

//
// set the viewportcoord.
//
void 
TViewport::setViewport( TFloat vXmin   , TFloat vYmin,
						TFloat vXlength, TFloat vYlength ) 
{
	viewXmin     = vXmin;
	viewYmin     = vYmin;;
	viewXlength  = vXlength;
	viewYlength  = vYlength;
	
	if (worldXlength != 0.0)
		scaleX = (TFloat)(viewXlength) / (worldXlength);
	else
		scaleX = 0.0;
	
	if (worldYlength != 0.0)
		scaleY = (TFloat)(viewYlength) / (worldYlength);
	else
		scaleY = 0.0;
}

//
// get the viewportborders
//
void 
TViewport::getViewport( TFloat& minX, TFloat& minY, 
						TFloat& maxX, TFloat& maxY ) const
{
	minX = viewXmin;
	minY = viewYmin;
	maxX = viewXmin + viewXlength;
	maxY = viewYmin + viewYlength;
}

//
// check if the point is in the window
//
bool 
TViewport::isInWin( TFloat x, TFloat y ) const {
	if ((worldXmin <= x) && (worldXmin+worldXlength >= x) &&
		(worldYmin <= y) && (worldYmin+worldYlength >= y))
		return (TRUE);
	else
		return (FALSE);
}
	
//
// check if the point is in the viewport
//
bool 
TViewport::isInView( TFloat x, TFloat y ) const {
	if ((viewXmin <= x) && (viewXmin+viewXlength >= x) &&
		(viewYmin <= y) && (viewYmin+viewYlength >= y))
		return (TRUE);
	else
		return (FALSE);
}

//
// copy operator
//
const TViewport& 
TViewport::operator=( const TViewport& viewPort ) 
{
	scaleX = viewPort.scaleX;
	scaleY = viewPort.scaleY;
	
	viewXmin = viewPort.viewXmin;
	viewYmin = viewPort.viewYmin;
	viewXlength = viewPort.viewXlength;
	viewYlength = viewPort.viewYlength;
	
	worldXmin = viewPort.worldXmin;
	worldYmin = viewPort.worldYmin;
	worldXlength = viewPort.worldXlength;
	worldYlength = viewPort.worldYlength;
	
	return (*this);
}

// transformation world -> viewport
const TFloat 
TViewport::viewX( TFloat x ) const 
{ 
	//	return ((scaleX * (x-worldXmin) + 0.5) + viewXmin);
	return ((scaleX * (x - worldXmin)) + viewXmin);
}
const TFloat 
TViewport::viewY( TFloat y ) const 
{ 
	//	return ((viewYlength-1+viewYmin) - (scaleY * (y-worldYmin) + 0.5));
	//	return ((TFloat)(scaleY * (y-worldYmin) + 0.5) + viewYmin);
	return ((scaleY * (y - worldYmin)) + viewYmin);
} 

// transformation viewport -> world
const TFloat 
TViewport::worldX( TFloat x ) const 
{ 
	return (((x - viewXmin) / scaleX) + worldXmin);
}
const TFloat 
TViewport::worldY( TFloat y ) const 
{ 
	return ((((viewYlength-1+viewYmin) - y) / scaleY) + worldYmin);
}
